package com.andframework.business;



import java.util.ArrayList;
import java.util.Map;

import com.andframework.common.KeyValue;
import com.andframework.config.FrameworkConfig;
import com.andframework.myinterface.UiCallBack;
import com.andframework.network.IHttpConnect;
import com.andframework.network.UploadFileVO;
import com.andframework.parse.BaseParse;

import android.os.AsyncTask;
import android.util.Log;

/**
 * @author 徐德良
 *基本业务对方，业务的抽象类
 */
public abstract class BaseBusi extends AsyncTask<String, String, BaseBusi>
{
	
	/**
	 * 业务数据,对应业务的解析类、数据结构
	 */
	protected BaseParse baseParse;
	
	/**
	 * 业务请求url，对应业务请求的url后面参数
	 */
	protected String  reqParam="";
    
	
    /**
     * 业务需要post非表单和文件上传数据，utf8格式，可以为空
     */
    public byte [] buffer;
    
    
    /**
     * 表单数据
     */
    private Map<String, String> map_normal;// = new HashMap<String, String>();
    
    /**
     * 上传文件数据，支持多文件上传
     */
    private Map<String, UploadFileVO> map_file;// = new HashMap<String, UploadFileVO>();
	
	
    //连接超时
    private int connectTimeout = 20000;
	
    //读取数据超时
	private int readTimeout = 20000;
	
    /**
     * 刷新业务数据的ui回接口
     */
    protected UiCallBack bcb;
    
    
    /**
     * 业务的主域名，来至配置文件，也可以通过实现修改
     */
    protected String domain = FrameworkConfig.getInst().getDomain();
    
    
    /**
     * 网络连接接口
     */
    private IHttpConnect iCon;
    
    
    private ArrayList<KeyValue> reqHeadPropertys = new ArrayList<KeyValue>();
    
    /**
     * 业务基类的构造函数
     * @param bCallBack  表示ui回调的实现
     */
    public BaseBusi(UiCallBack bCallBack,Class<?> cls)
    {
    	bcb = bCallBack;
    	if(cls==null)
    	{
    		Log.e("BaseBusi", "Class<?> is null");
    	}
    	else
    	{
    		try 
        	{
        		 Object obj =cls.newInstance();
        		 if(obj instanceof BaseParse)
        		 {
        			 baseParse=(BaseParse)obj;
        		 }
        		 else
        		 {
        			 Log.e("BaseBusi", "newInstance not extends BaseParse ");
        		 }
    		} 
        	catch (IllegalAccessException e) 
        	{
    			// TODO Auto-generated catch block
    			e.printStackTrace();
    		} 
        	catch (InstantiationException e) 
        	{
    			e.printStackTrace();
    		}
    	}
    	
    }
    
    //设置连接超时
	public void setConnectTimeout(int connectTimeout)
	{
		this.connectTimeout = connectTimeout;
	}
	
	//设置读取超时
	public void setReadTimeout(int readTimeout)
	{
		this.readTimeout = readTimeout;
	}
	
    /**
     * 设置表单数据
     */
	public void setFormData( Map<String, String> map_normal)
	{
		this.map_normal=map_normal;
	}

	
	/**
     * 设置上传文件数据
     */
	public void setUploadFileData( Map<String, UploadFileVO> map_file)
	
	{
		this.map_file=map_file;
	}
	
	
	/**
     * 设置http协议头
     */
    public void setHeadPropertys(ArrayList<KeyValue> kvs)
	{
		if(null!=kvs && kvs.size()>0)
		{
			reqHeadPropertys=kvs;
		}
	}
    
	/**
     * 设置业务需要的协议头
     */
    public void pushHeadPropertys(KeyValue kv)
	{
		if(kv.getKey()!=null && !kv.getKey().equals("")&&kv.getValue()!=null && !kv.getValue().equals(""))
		{
			reqHeadPropertys.add(kv);
		}
	}
    /**
     * 处理业务之前调用的准备方法
     * @return boolean false 不执行该业务
     */
    protected abstract void prepare();
    
    
    /**
     * @return 获取返回的业务数据结构
     */
    public BaseParse getBaseStruct()
    {
    	return baseParse;
    }
    
    
    /**
     * 取消业务处理
     */
    public void iCancle()
    {
    	this.cancel(true);
    	if(null!=iCon)
    	{
    		iCon.disConnect();
    	}
    }
    
    /**
     * 执行网络请求及处理
     */
    public void iExecute()
    {
    	prepare();
    	if(null==domain ||domain.length()<1)
    	{
    		Log.i("iExecute", "domain=kong");
    		return;
    	}
    	this.execute(domain+reqParam);
    	Log.i("iExecute", "url="+domain+reqParam);
    }
    
	/* (non-Javadoc)
	 * @see android.os.AsyncTask#doInBackground(Params[])
	 */
    
    //开放出去，不仅仅局限于网络操作,抽象具体的任务类，在可以注入任务在后台执行
	@Override
	protected BaseBusi doInBackground(String... params) //
	{
		
		String url = params[0];
		if(FrameworkConfig.getInst().getConfigValue("log")!=null||"true".equals(FrameworkConfig.getInst().getConfigValue("log")))
		Log.i("busi Req url=", url);
		iCon =  new IHttpConnect();
		
		iCon.setFormData(map_normal);
		iCon.setUploadFileData(map_file);
		iCon.setConnectTimeout(connectTimeout);
		iCon.setReadTimeout(readTimeout);
		if(reqHeadPropertys.size()>=0)
		{
			for(int i=0;i<reqHeadPropertys.size();i++)
			{
				iCon.pushHeadPropertys(reqHeadPropertys.get(i));
			}
		}
		
		iCon.sendData(buffer, url);
		byte[] result = iCon.receiveData();
		if(FrameworkConfig.getInst().getConfigValue("log")!=null&&"true".equals(FrameworkConfig.getInst().getConfigValue("log")))
		{
			if(null!=result)
			{
				Log.i(url+" busi rsp =", new String(result));
			}
			else
			{
				Log.i(url+" busi rsp =", "null");
			}
		}
		
		
		if(null!=baseParse)
		{
			baseParse.parse(result);//.parseResult(result);
			baseParse.httpResponseCode=iCon.httpResponseCode();
		}
		
		return this;
		
	}

	/* (non-Javadoc)
	 * @see android.os.AsyncTask#onCancelled()
	 * Runs on the UI thread after cancel(boolean) is invoked.
	 */
	@Override
	protected void onCancelled() 
	{
		super.onCancelled();
		
		Log.i("onCancelled>", "");
		
		//添加http请求取消
		
	}

	/* (non-Javadoc)
	 * @see android.os.AsyncTask#onPostExecute(java.lang.Object)
	 * Runs on the UI thread after doInBackground(Params...). doInBackground返回的结果
	 */
	@Override
	protected void onPostExecute(BaseBusi object) 
	{
		bcb.uiCallBack(object);
	}

	/* (non-Javadoc)
	 * @see android.os.AsyncTask#onPreExecute()
	 * Runs on the UI thread before doInBackground(Params...).
	 */
	@Override
	protected void onPreExecute() 
	{
		// 任务启动，可以在这里显示一个对话框，这里简单处理
	  //  message.setText(R.string.task_started);
	}

	
	/* (non-Javadoc)
	 * @see android.os.AsyncTask#onProgressUpdate(Progress[])
	 * Runs on the UI thread after publishProgress(Progress...) is invoked.
	 */
	@Override
	protected void onProgressUpdate(String... values) 
	{
		// 更新进度
	}
	
	//publishProgress(Progress... values)
	//This method can be invoked from doInBackground(Params...) to publish updates on the UI thread while the background computation is still running.

}
