package com.andframework.business;

import java.util.Iterator;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.app.Activity;
import android.content.Context;
import android.telephony.CellLocation;
import android.telephony.NeighboringCellInfo;
import android.telephony.TelephonyManager;
import android.telephony.cdma.CdmaCellLocation;
import android.telephony.gsm.GsmCellLocation;
import android.util.Log;

import com.andframework.business.BaseBusi;
import com.andframework.myinterface.UiCallBack;
import com.andframework.parse.BaseParse;
import com.andframework.parse.GetLocaionParse;

/**
 * @author 徐德良
 */
public class GetLocationBusi extends BaseBusi
{
	Activity act;
	//public String sendCon;
	public GetLocationBusi(UiCallBack bCallBack)
	{
		super(bCallBack,GetLocaionParse.class);
	}
	
	public void setActivity(Activity act)
	{
		this.act = act;
	}
	
	/**
	 * 是否可以符合执行该任务条件
	 */
	public boolean isCanExecute()
	{
		if(null==act)
		{
			Log.i("iExecute", "activity is null");
			return false;
		}
		TelephonyManager telMan = (TelephonyManager) act
				.getSystemService(Context.TELEPHONY_SERVICE);
		String mcc, mnc;
		int cid, lac;
		String operator = telMan.getNetworkOperator();
		boolean isCdma = false;
		CellLocation cl = telMan.getCellLocation();
		if (cl instanceof GsmCellLocation)
		{
			cid = ((GsmCellLocation) cl).getCid();
			lac = ((GsmCellLocation) cl).getLac();
			mcc = operator.substring(0, 3);
			mnc = operator.substring(3);
		}
		else if (cl instanceof CdmaCellLocation)
		{
			cid = ((CdmaCellLocation) cl).getBaseStationId();
			lac = ((CdmaCellLocation) cl).getNetworkId();
			
			mcc = operator.substring(0, 3);
			int systemId = ((CdmaCellLocation) cl).getSystemId();
			mnc = String.valueOf(systemId);
			isCdma = true;
		}
		else
		{
			Log.i("iExecute", "CellLocation is unknow type");
			return false;
		}
		return true;
	}
	
	/*@Override
	protected BaseParse parseResult(byte[] data)
	{
		GetLocaionParse p = new GetLocaionParse();
		p.Parse(data);
		return p;
	}*/
	
	public void iExecute()
	{
		try
		{
			if(null==act)
			{
				Log.i("iExecute", "activity is null");
				return;
			}
			TelephonyManager telMan = (TelephonyManager) act
					.getSystemService(Context.TELEPHONY_SERVICE);
			String mcc, mnc;
			int cid, lac;
			String operator = telMan.getNetworkOperator();
			boolean isCdma = false;
			CellLocation cl = telMan.getCellLocation();
			if (cl instanceof GsmCellLocation)
			{
				cid = ((GsmCellLocation) cl).getCid();
				lac = ((GsmCellLocation) cl).getLac();
				mcc = operator.substring(0, 3);
				mnc = operator.substring(3);
			}
			else if (cl instanceof CdmaCellLocation)
			{
				cid = ((CdmaCellLocation) cl).getBaseStationId();
				lac = ((CdmaCellLocation) cl).getNetworkId();
				
				mcc = operator.substring(0, 3);
				int systemId = ((CdmaCellLocation) cl).getSystemId();
				mnc = String.valueOf(systemId);
				isCdma = true;
			}
			else
			{
				Log.i("iExecute", "CellLocation is unknow type");
				return ;
			}
			
			JSONObject tower = new JSONObject();
			try
			{
				tower.put("cell_id", cid);
				tower.put("location_area_code", lac);
				tower.put("mobile_country_code", mcc);
				tower.put("mobile_network_code", mnc);
			}
			catch (JSONException e)
			{
				Log.e("requestNetworkLocation", "JSONObject put failed", e);
			}
			
			JSONArray jarray = new JSONArray();
			jarray.put(tower);
			
			List<NeighboringCellInfo> ncis = telMan.getNeighboringCellInfo();
			Iterator<NeighboringCellInfo> iter = ncis.iterator();
			
			NeighboringCellInfo nci;
			JSONObject tmpTower;
			while (iter.hasNext())
			{
				nci = iter.next();
				tmpTower = new JSONObject();
				try
				{
					tmpTower.put("cell_id", nci.getCid());
					tmpTower.put("location_area_code", nci.getLac()); // 1.6不支持这个函数，需要单另打包。
					tmpTower.put("mobile_country_code", mcc);
					tmpTower.put("mobile_network_code", mnc);
				}
				catch (JSONException e)
				{
					Log.e("JSONException", "JSONObject put failed");
				}
				jarray.put(tmpTower);
			}
			
			JSONObject object;
			if (!isCdma)
				object = createJSONObject("cell_towers", jarray);
			else
				object = createCDMAJSONObject("cell_towers", jarray, mcc, mnc);
			
			//sendCon=object.toString();
			this.buffer=object.toString().getBytes();
			this.domain="http://www.google.com/loc/json";
			super.iExecute();
			
		}
		catch (Exception e)
		{
			Log.d("Exception",
					"request GsmCellLocation failed,reason:" + e.getMessage());
		}
	}
	
	private JSONObject createJSONObject(String arrayName, JSONArray array)
	{
		JSONObject object = new JSONObject();
		try
		{
			object.put("version", "1.1.0");
			object.put("host", "maps.google.com");
			object.put(arrayName, array);
		}
		catch (JSONException e)
		{
			Log.e("JSONException", "JSONObject put failed");
		}
		return object;
		
	}
	
	// 组装cdma的json数据
	private JSONObject createCDMAJSONObject(String arrayName, JSONArray array,
			String mcc, String mnc)
	{
		
		JSONObject object = new JSONObject();
		try
		{
			object.put("version", "1.1.0");
			object.put("host", "maps.google.com");
			object.put("home_mobile_country_code", mcc);
			object.put("home_mobile_network_code", mnc);
			object.put("radio_type", "cdma");
			object.put("request_address", true);
			if ("460".equals(mcc))
				object.put("address_language", "zh_CN");
			else
				object.put("address_language", "en_US");
			
			object.put(arrayName, array);
		}
		catch (JSONException e)
		{
			Log.e("createCDMAJSONObject>JSONException", "JSONObject put failed");
		}
		return object;
	}

	@Override
	protected void prepare() {
		// TODO Auto-generated method stub
	}
}
