/**
 * 
 */
package com.andframework.business;

import java.util.Vector;

import com.andframework.network.IHttpConnect;

import android.content.Context;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.util.Log;


/**
 * @author xudeliang
 * 图片异步下载任务管理器
 * 考虑到图片显示的优先级是最后加入的级别较高，也就是当前界面显示的图片。
 * 当从界面1，快速切换3个界面回到界面1，那么下载优先是4321
 * 
 * 一：对图片请求进行时间戳排序，默认后加进来的图片优先级高
 * ：
　　二：在Wifi网络环境下，各种网络操作可以并行运行 4个任务，　在GPRS网络环境下 2个任务
   三、控制无限个图片异步请求
 */
/**
 * @author xudeliang
 *
 */
public class PicDownTaskManager extends Thread implements FinishedListner
{

	private static PicDownTaskManager pdtm;
	
	
	//任务列表
	private Vector<LoadImageTsk> runningQueue;
	
	private Vector<LoadImageTsk> readyQueue;
	private boolean isRunning=true;
	private PicDownTaskManager()
	{
		runningQueue=new Vector<LoadImageTsk>();
		readyQueue=new Vector<LoadImageTsk>();
	}
	
	public static PicDownTaskManager getInst()
	{
		if(pdtm==null)
		{
			pdtm = new PicDownTaskManager();
			pdtm.start();
		}
		return pdtm;
	}
	
	public void destroyPicDownTask()
	{
		pdtm=null;
		isRunning = false;
		runningQueue.removeAllElements();
		readyQueue.removeAllElements();
	}
	
	//boolean iswait=true;
	public void postTask(LoadImageTsk msg)
	{
		synchronized(this)
		{
			msg.timestmp = System.currentTimeMillis();
			msg.setFinishedListner(this);
			runningQueue.add(msg);
			if(readyQueue.size()==0)
			{
				notify();
			}
		}
		
	}
	


	//计算图片进去优先级，
	//场景1、由一个图片列表界面进入另一个图列表界面。找出间隔先请求后进来的图片
	//对时间戳排序，找出一组，优先后面一组图片下载
	//200毫秒间隔为单位
	private int calcIndex()
	{
		int size = runningQueue.size();
		long temtimestamp=0;//上次的时间戳
		int blank=0;//前后时间间隔
		int returnindex=-1;//返回索引
		for(int i=size-1;i>=0;i--)//后遍历
		{
			LoadImageTsk lt = runningQueue.get(i);
			
			returnindex=i;
			if(i==size-1)
			{
				temtimestamp=lt.timestmp;
			}
			else
			{
				if(blank==0)
				{
					blank = (int)(temtimestamp-lt.timestmp);//第一次时间间隔
					temtimestamp=lt.timestmp;
					if(blank>200)
					{
						return returnindex+1;
					}
				}
				else if(blank+200<(temtimestamp-lt.timestmp))//找到间隔点
				{
					//Log.i("splitUrl:", lt.picUrl);
					return returnindex+1;
				}
				else//没找到，优先先进来的
				{
					temtimestamp = lt.timestmp;
				}
				
			}
		}
		
		return returnindex;
	}
	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	@Override
	public void run()
	{
		while(isRunning)
		{
			//获取最需要先处理的图片
			synchronized(this)
			{
				int tsknum = 2;
				int nettype = IHttpConnect.getCurrentAPNType();
				if(nettype==ConnectivityManager.TYPE_WIFI)
				{
					tsknum=4;
				}
				
				if(readyQueue.size()>tsknum)//4个任务在跑
				{
					try
					{
						wait();
					}
					catch (InterruptedException e)
					{
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					
				}
				else
				{
					int downindex=calcIndex();
					if(downindex==-1)
					{
						try
						{
							wait();
						}
						catch (InterruptedException e)
						{
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					}
					else
					{
						LoadImageTsk  lit = runningQueue.remove(downindex);//
						readyQueue.add(lit);
						lit.iExcute();
						//Log.i("excute>", lit.picUrl);
					}
					
				}
			}
			
			try
			{
				Thread.sleep(500);//让出500毫秒给其他线程抢
			}
			catch (InterruptedException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
		}
	}

	/* (non-Javadoc)
	 * @see com.andframework.ui.FinishedListner#onFinished(com.andframework.ui.LoadImageTsk)
	 */
	@Override
	public void onFinished(LoadImageTsk lt)
	{
		// TODO Auto-generated method stub
		synchronized(this)
		{
			readyQueue.remove(lt);
			this.notify();
			
		}
		
	}

	/* (non-Javadoc)
	 * @see com.andframework.ui.FinishedListner#onCancle(com.andframework.ui.LoadImageTsk)
	 */
	@Override
	public void onCancle(LoadImageTsk lt)
	{
		synchronized(this)
		{
			readyQueue.remove(lt);
			this.notify();
		}
	
		// TODO Auto-generated method stub
		
	}
}
