package com.andframework.common;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;

public class CustomNotify
{
	
	private Notification n = null;
	
	private NotificationManager nm = null;
	
	private Context activity = null;
	
	/**
	 * @activity 调用提醒的活动窗口
	 * @iconRes 提醒的小图标
	 * @isVibrate 是否振动
	 * @soundUri 提醒声音路径
	 */
	public CustomNotify(Context activity, int iconRes, boolean isVibrate,
			Uri soundUri)
	{
		
		this.activity = activity;
		nm = (NotificationManager) activity
				.getSystemService(android.content.Context.NOTIFICATION_SERVICE);
		// 实例化Notification
		n = new Notification();
		// 设置显示图标，该图标会在状态栏显示
		n.icon = iconRes;
		if (isVibrate)
		{
			// after a 100ms delay, vibrate for 250ms, pause for 100 ms and
			// then vibrate for 500ms.
			n.vibrate = new long[] { 100, 250, 100, 500 };
		}
		if (null != soundUri)
		{
			n.sound = soundUri;
		}
		// 自动取消，点击或者清除都会去掉
		n.flags = Notification.FLAG_AUTO_CANCEL;
	}
	
	/**
	 * @notifyActivity 点击提醒弹出的窗口
	 * @title 下拉提醒的标题
	 * @content 下拉提醒看到的内容
	 * @notifyId 提醒的id
	 * @tickerText 提醒状态栏未下拉显示的文本
	 */
	public void showNotification(Class<?> cls, String title, String content,
			int notifyId, String tickerText)
	{
		// 实例化Intent
		Intent intent = new Intent(activity, cls);
		// 获取PendingIntent
		PendingIntent pi = PendingIntent.getActivity(activity, 0, intent, 0);
		// 设置事件信息
		n.setLatestEventInfo(activity, title, content, pi);
		// 显示时间
		n.when = System.currentTimeMillis();
		// 设置显示提示信息，该信息也会在状态栏显示
		n.tickerText = tickerText;
		// 发出通知
		/*
		 * Notification.Builder bl = new Notification.Builder(this);
		 * bl.setContentTitle("My Title"); bl.setContentText("My Content");
		 * bl.setContentIntent(pi); n= bl.getNotification();
		 */
		nm.notify(notifyId, n);
		
	}
	
	public void cancleNotify(int notifyId)
	{
		nm.cancel(notifyId);
	}
}
