package com.andframework.config;

import java.io.IOException;
import java.util.ArrayList;
import org.xmlpull.v1.XmlPullParserException;
import android.content.Context;
import android.content.res.XmlResourceParser;
import android.util.Log;
import com.andframework.common.KeyValue;


/**
 * @author 徐德良
 *该类对应frameworkconfig.xml
 */
public class FrameworkConfig 
{
	
	
	/**
	 * 日志路径
	 */
	private static String LOGPATH = "";
	
	/**
	 * 日志文件名称
	 */
	private static String LOGNAME = "";
	
	/**
	 * 域名domain
	 */
	private static String DOMAIN ="";
	
	/**
	 * 单例模式
	 */
	private static FrameworkConfig fConfig;
	
	/**
	 * 应用上下文
	 */
	private static Context context;
	
	/**
	 * 保存key value
	 */
	private static ArrayList<KeyValue> configValue = new ArrayList<KeyValue>();
	/**
	 * 单例模式构造函数
	 */
	private FrameworkConfig(Context c,int R_xml_config)
	{
		context=c;
		buildConfigData(context,R_xml_config);
	}
	
	
	/**
	 * @return 单例模式，执行之前确保执行getInst(Context context)
	 */
	public static FrameworkConfig getInst()
	{
		if(null ==fConfig)
		{
			fConfig = new FrameworkConfig(null,-1);
		}
		
		return fConfig;
	}
	
	
	/**
	 * @return 单例模式，至少执行一次该方法
	 */
	public static FrameworkConfig getInst(Context context,int R_raw_zmconfig)
	{
		if(null ==fConfig)
		{
			fConfig = new FrameworkConfig(context,R_raw_zmconfig);
		}
		
		return fConfig;
	}
	
	/**
	 * @return  返回日志路径
	 */
	public String getLogPath()
	{
		return LOGPATH;
	}
	
	/**
	 * @return  返回日志文件名称
	 */
	public String getLogName()
	{
		return LOGNAME;
	}
	
	
	/**
	 * @return 返回主域名
	 */
	public String getDomain()
	{
		return DOMAIN;
	}
	
	
	/**
	 * 获取配置文件的值
	 * @param key 配置的key
	 */
	public String getConfigValue(String key)
	{
		String value = null;
		if(null!=key && !key.equals(""))
		{
			int size = configValue.size();
			for(int i=0;i<size;i++)
			{
				KeyValue kv = configValue.get(i);
				if(key.equals(kv.getKey()))
				{
					value=kv.getValue();
					break;
				}
			}
		}
		return value;
	}
	
	/**
	 * 解析raw的config.xml文件
	 * @param c 应用的上下文
	 */
	
	private void buildConfigData(Context c,int R_xml_config)
	{
		if(c==null)return;
		XmlResourceParser xrp=  c.getResources().getXml(R_xml_config);
		try {
			xrp.next();
			int eventType = xrp.getEventType();
			boolean startTag=false;
			String tagname=null;
			while (eventType != XmlResourceParser.END_DOCUMENT)  {
			    if(eventType == XmlResourceParser.START_DOCUMENT)  {
			    	
			    } 
			    else if(eventType == XmlResourceParser.START_TAG) {
			    	startTag=true;
			    	tagname=xrp.getName();
			    } 
			    else if(eventType == XmlResourceParser.END_TAG) {
			    	
			    } 
			    else if(eventType == XmlResourceParser.TEXT) {
			    	if(startTag){
				    	 String   value = xrp.getText();
				    	 Log.i(tagname, value+"");
				    	 KeyValue kv = new KeyValue(tagname,value);
				    	 configValue.add(kv);
				    	 startTag=false;
				    	 if(tagname.equals("domain")){
				    		 DOMAIN=value;
				    	 }
				    	 if(tagname.equals("logpath")){
				    		 LOGPATH=value;
				    	 }
				    	 if(tagname.equals("logname")){
				    		 LOGNAME=value;
				    	 }
			    	}
			    	
			    }
			    eventType = xrp.next();
			 }
		} catch (XmlPullParserException e1) {
			e1.printStackTrace();
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		finally {
			xrp.close();
		}
	}
	
	/**
	 * 设置配置文件的值
	 * @param key 配置的key
	 * @return value
	 */
	public String setValue(String key,String Value)
	{
		if(null!=key && !key.equals(""))
		{
			int size = configValue.size();
			for(int i=0;i<size;i++)
			{
				KeyValue kv = configValue.get(i);
				if(key.equals(kv.getKey()))
				{
					kv.setValue(Value);
					return Value;
				}
			}
		}
		return null;
	}
}
