package com.andframework.network;
import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.InetSocketAddress;
import java.net.Proxy;
import java.net.URL;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.zip.GZIPInputStream;

import org.apache.http.client.ClientProtocolException;

import com.andframework.common.KeyValue;


import android.app.Activity;
import android.content.Context;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.util.Log;

/**
 * @author 徐德良
 *该类负责数据发送、接受，接入点自动适配。
 * 负责下载1m以下的数据，不建议大数据下载
 */
public class IHttpConnect
{
	
	private int httpResponseCode=-1; 
	//会话
	private static String session="";
	
	public static Context  mainActivity;
	
	private HttpURLConnection connection;
	
	private int connectTimeout = 20000;
	
	private int readTimeout = 20000;
	
	//DefaultHttpClient httpClient;
	//HttpResponse response;//
	private static String ENCODING = "UTF-8";// 字符编码
	private String boundary = "----" + System.currentTimeMillis();// 数据分隔符
	
	static
	{
		//确保多个异步请求导致 getResponseCode 返回-1
		System.setProperty("http.keepAlive", "false");
		//byte[]cookies = ZMFile.readAll("app");//cookie保存在手机应用目录下
		//cookie = new String(cookies);
	}
	/**
	 * http协议头key value
	 */
	private ArrayList<KeyValue> reqHeadPropertys = new ArrayList<KeyValue>();
	
	
	/**
     * 表单数据
     */
    private Map<String, String> map_normal ;//= new HashMap<String, String>();
    
    /**
     * 上传文件数据，支持多文件上传
     */
    private Map<String, UploadFileVO> map_file;// = new HashMap<String, UploadFileVO>();
    
    /**
     * 代理host
     */
    private static String proxyHost=null;
    
    /**
     * 代理port
     */
    private static int proxyPort=0;
	
	
	public IHttpConnect()
	{
		
	}
	
	//设置连接超时
	public void setConnectTimeout(int connectTimeout)
	{
		this.connectTimeout = connectTimeout;
	}
	
	//设置读取超时
	public void setReadTimeout(int readTimeout)
	{
		this.readTimeout = readTimeout;
	}
	
	/**
     * 设置表单数据
     */
	public void setFormData( Map<String, String> map_normal)
	{
		this.map_normal=map_normal;
	}

	
	/**
     * 设置上传文件数据
     */
	public void setUploadFileData( Map<String, UploadFileVO> map_file)
	{
		this.map_file=map_file;
	}
	
	/**
	 * @param kv http请求头
	 */
	public void pushHeadPropertys(KeyValue kv)
	{
		if(kv.getKey()!=null && !kv.getKey().equals("")&&kv.getValue()!=null && !kv.getValue().equals(""))
		{
			reqHeadPropertys.add(kv);
		}
	}

	public void disConnect()
	{
		if(null!=connection)
		{
			connection.disconnect();
		    connection=null;
		}
	}
	
	/**
	 * 设置代理
	 * 
	 */
	
	public static void setProxy(String iproxyHost,int iproxyPort)
	{
		proxyHost=iproxyHost;
		proxyPort=iproxyPort;
	}
	
	
	/**
	 * 获取当前网络类型 -1表示没联网或者mainactivity 为null
	 * @return  返回网络联网类型 ConnectivityManager.TYPE_WIFI等
	 */
	public static int getCurrentAPNType()
	{
		if(null!=mainActivity )//查找是否有网络及是否是wap代理上网，获取代理
		{
			ConnectivityManager connectivityManager = (ConnectivityManager) mainActivity.getSystemService(Context.CONNECTIVITY_SERVICE);
			if (connectivityManager != null) 
			{
				NetworkInfo info = connectivityManager.getActiveNetworkInfo();
				if (info != null && info.isAvailable())
				{
					return info.getType();
				}
			}
		}
		return -1;
	}
	
	
	/**
	 * @param buffer 发送的数据，可以为空，为空表示get方式 否则是post,
	 * @param url 获取数据的url
	 * @return 发送成功返回true，否则false
	 */
	public boolean sendData(byte[] buffer,String url)
	{
		
		
		if(null!=mainActivity&& null==proxyHost)//查找是否有网络及是否是wap代理上网，获取代理
		{
			ConnectivityManager connectivityManager = (ConnectivityManager) mainActivity.getSystemService(Context.CONNECTIVITY_SERVICE);
			if (connectivityManager != null) 
			{
				NetworkInfo info = connectivityManager.getActiveNetworkInfo();
				if (info != null && info.isAvailable())
				{
					int nettype=info.getType();
					if (nettype != ConnectivityManager.TYPE_WIFI)//非wifi
					{
						proxyHost = android.net.Proxy.getDefaultHost();
						proxyPort = android.net.Proxy.getDefaultPort();
					}
					//用户选择apn为wap但是没设置代理
					if((null==proxyHost || proxyHost.length()<=0) &&("cmwap".equalsIgnoreCase(info.getExtraInfo()) ||"3gwap".equalsIgnoreCase(info.getExtraInfo())||"uniwap".equalsIgnoreCase(info.getExtraInfo()) ))
					{
						proxyHost="10.0.0.172";
						proxyPort=80;
					}
					
					//oms手机 默认cmnet 也会获取代理
					if("cmnet".equalsIgnoreCase(info.getExtraInfo()) ||"3gnet".equalsIgnoreCase(info.getExtraInfo())||"uninet".equalsIgnoreCase(info.getExtraInfo()))
					{
						proxyHost=null;
						proxyPort=0;
					}
				}
				else
				{
					return false;
				}

			}
			else
			{
				return false;
			}
		}
		
		try
		{
			URL postUrl = new URL(url);
	        
	        if(proxyHost != null && proxyHost.trim().length() > 0)
	        {
				Proxy proxy = new Proxy(Proxy.Type.HTTP, new InetSocketAddress(proxyHost, proxyPort));
				connection = (HttpURLConnection) postUrl.openConnection(proxy);
			} else 
			{
				connection = (HttpURLConnection) postUrl.openConnection();
			}
			connection.setConnectTimeout(connectTimeout);
			connection.setReadTimeout(readTimeout);
			connection.setUseCaches(false);
			
			connection.setDoInput(true);
			connection.setRequestProperty("Charset", ENCODING);// 设置请求的编码方式
			connection.setRequestProperty("Accept-Encoding","gzip, deflate");//设置压缩请求
			if (null != session &&!session.equals("")) 
			{
				connection.setRequestProperty("Cookie", session);
			}
	      
	        //设置业务需要的http头参数
	        if(reqHeadPropertys.size()>0)
			{
				int reqPs  = reqHeadPropertys.size();
				for(int i=0;i<reqPs;i++)
				{
					KeyValue kv = reqHeadPropertys.get(i);
					connection.setRequestProperty(kv.getKey(), kv.getValue());
				}
			}
	        
			if(buffer!=null )//仅仅post 二进制数据
			{
				//connection.setRequestProperty("Content-Type","application/octet-stream");
				connection.setDoOutput(true);
				connection.setRequestMethod("POST");
				connection.connect();
				connection.getOutputStream().write(buffer);
				connection.getOutputStream().flush();
				connection.getOutputStream().close();
			}
			else if(null!=map_file && map_file.size()>0)//提交文件及表单数据
			{
				connection.setDoOutput(true);
				connection.setRequestProperty("Content-Type", "multipart/form-data; boundary=" + boundary);
				connection.setRequestMethod("POST");
				connection.connect();
				this.postByMultipart(connection.getOutputStream(), map_normal, map_file);
			}
			else if(null!=map_normal && map_normal.size()>0)//提交表单数据
			{
				connection.setDoOutput(true);
				connection.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
				connection.setRequestMethod("POST");
				connection.connect();
				this.postByFormUrlencoded(connection.getOutputStream(), map_normal);
			}
			else 
			{
				connection.setRequestMethod("GET");
				connection.connect();
			}
			
			
			return true;
		}
		catch (ClientProtocolException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
			Log.i("ClientProtocolException", "url:"+url);
		}
		catch (IOException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
			Log.i("IOException", "url:"+url);
		}
		connection=null;
		return false;
	}
	
	/**
	 * 
	 * 重置session 
	 */
	public static void resetSession()
	{
		session="";
	}
	
	/**
	 * 获取session
	 * @return
	 */	
	public static String getSession() {
		return session;
	}

	
	/**
	 * 获取httpResponseCode
	 * @return
	 */	
	public  int httpResponseCode() {
		return httpResponseCode;
	}
	
	/**
	 * @return 返回请求回来的数据包
	 * @throws IOException 
	 */
	public byte[] receiveData()
	{
		try
		{
			if(null==connection)
			{
				return null;
			}
			
			int rc = connection.getResponseCode();
			httpResponseCode=rc;
            if (rc == HttpURLConnection.HTTP_OK) 
            {
            	// 处理回应的协议头：cookie
        		if (null == session || session.equals(""))
        		{
        			String tmpsession = "";
        			Map<String, List<String>> m = connection.getHeaderFields();
        			List<String> list = m.get("Set-Cookie");
        			if (null != list) 
        			{
        				for (String s : list) 
        				{
        					if(s.endsWith(";"))
        					{
        						tmpsession += s;
        					}
        					else
        					{
        						tmpsession += s + ";";
        					}
        					///String k = s.substring(0, s.indexOf(";"));
        					//tmpsession += s + ";";
        				}
        			}
        			if(!tmpsession.equals(""))
        			{
        				session = tmpsession;
        			}
        			tmpsession = "";
        			list = m.get("set-cookie");
        			if (null != list) 
        			{
        				for (String s : list) 
        				{
        					if(s.endsWith(";"))
        					{
        						tmpsession += s;
        					}
        					else
        					{
        						tmpsession += s + ";";
        					}
        					//String k = s.substring(0, s.indexOf(";"));
        					//tmpsession += k + ";";
        				}
        			}
        			if(!tmpsession.equals(""))
        			{
        				session = session+tmpsession;
        			}
        		}
            	
            	InputStream is = connection.getInputStream();
        		String encoding = connection.getContentEncoding();
				BufferedInputStream inputStream = null;
				if(encoding!=null && encoding.equals("gzip"))
				{
					inputStream = new BufferedInputStream(new GZIPInputStream(is),8*1024);
				} 
				else 
				{
					inputStream = new BufferedInputStream(is,8*1024);
				}
            	ByteArrayOutputStream bos = new ByteArrayOutputStream();
            	
        	    byte[] buffer = new byte[8*1024];
        	    int bufsize = inputStream.read(buffer);
        	    while(bufsize!=-1)
        	    {
        		    bos.write(buffer,0,bufsize);
        		    bufsize = inputStream.read(buffer);
        	    }
        	    buffer = bos.toByteArray();
        	    bos.close();
        	   
        	    if(null!=inputStream)
        	    {
        	    	inputStream.close();
        	    	inputStream=null;
        	    }
        	    
        	    if(null!=is)
        	    {
        	    	is.close();
        	    	is=null;
        	    }
        	    connection.disconnect();
        	    connection=null;
        	    return buffer;
            }
            else
            {
            	Log.i("NetSender>receiveData>ResponseCode：", rc+"");
            	//connection.get
            }
		}
		catch(IOException e)
		{
			Log.i("NetSender>receiveData> IOException:", "message:"+e.getMessage());
			e.printStackTrace();
			//Util.printLog("NetSender>receiveData> IOException：", e.getMessage()+"", 0, 0);
		}

		return null;
			
	}
	
	/**
	 * 以x-www-form-urlencoded方式提交，合成协议体
	 * 
	 * @param out
	 *            输出流
	 * @param map_normal
	 *            普通表单字段map
	 * @throws IOException
	 */
	private void postByFormUrlencoded(OutputStream out, Map<String, String> map_normal) throws IOException
	{
		// 合成POST参数
		StringBuffer sb = new StringBuffer();
		for (String key : map_normal.keySet()) 
		{
			sb.append("&");
			sb.append(key);
			sb.append("=");
			sb.append(URLEncoder.encode(map_normal.get(key).toString(), ENCODING));
		}
		String postData = sb.substring(1);
		byte[] b = postData.getBytes();
		out.write(b, 0, b.length);
		out.flush();
		out.close();
	}
	
	/**
	 * 以Multipart方式提交，附带文件提交，合成协议体
	 * 
	 * @param out
	 *            输出流
	 * @param map_normal
	 *            普通表单字段map
	 * @param map_file
	 *            文件表单字段map
	 * @throws UnsupportedEncodingException
	 * @throws IOException
	 */
	private void postByMultipart(OutputStream out, Map<String, String> map_normal, Map<String, UploadFileVO> map_file) throws UnsupportedEncodingException, IOException
	{

		// 普通的表单字段内容
		if(null!=map_normal && map_normal.size()>0)
		{
			StringBuilder sb = new StringBuilder();
			for (String key : map_normal.keySet())
			{
				sb.append("--");
				sb.append(boundary);
				sb.append("\r\nContent-Disposition: form-data; name=\"");
				sb.append(key);
				sb.append("\"\r\n\r\n");
				sb.append(map_normal.get(key));
				sb.append("\r\n");
			}
			out.write(sb.toString().getBytes(ENCODING));
			out.flush();
		}
		

		// 上传文件数据
		byte[] buffer = new byte[1024];
		for (String key : map_file.keySet())
		{
			UploadFileVO vo = map_file.get(key);
			StringBuilder split = new StringBuilder();
			split.append("--");
			split.append(boundary);
			split.append("\r\nContent-Disposition: form-data; name=\"");
			split.append(key);
			split.append("\"; filename=\"");
			split.append(vo.getFileName());
			split.append("\"\r\nContent-Type: ");
			split.append(vo.getContentType());
			split.append("\r\n\r\n");
			out.write(split.toString().getBytes(ENCODING));
			// 写入文件字节体
			InputStream inputStream = vo.getInputStream();
			if (null != inputStream) 
			{
				int len = 0;
				while ((len = inputStream.read(buffer)) != -1) 
				{
					out.write(buffer, 0, len);
				}
				inputStream.close();
			}

			out.write("\r\n".getBytes(ENCODING));
			out.flush();
		}

		// 写结尾
		out.write(("--" + boundary + "--\r\n").getBytes(ENCODING));

		out.flush();
		out.close();
	}
}
