package com.andframework.parse;

import java.io.ByteArrayInputStream;
import java.io.UnsupportedEncodingException;

import org.json.JSONException;
import org.json.JSONObject;
import org.w3c.dom.Document;


/**
 * @author 徐德良 
 * 数据结构解析的基类
 *
 */
public  class BaseParse 
{

	/**
	 * 对应http返回的错误吗
	 */
	public int httpResponseCode;
	
	/**
	 * 解析方式 json
	 */
	public static final int JSON = 0;
	
	/**
	 * 解析方式xml
	 */
	public static final int XML = 1;
	
	/**
	 * 二进制 不解析
	 */
	public static final int Binary = 2;
	
	/**
	 * 解析类型，默认是json
	 */
	protected int parseType = JSON;
	
	/**
	 * 如果是json解析，该对象有效
	 */
	public JSONObject jsonObject;
	
	/**
	 * 如果xml解析，该对象有效
	 */
	public Document doc;
	
	/**
	 * 错误码 根据实际协议
	 */
	public int errorCode;
	
	/**
	 * 提示信息 根据实际协议
	 */
	public String message;
	
	
	/**
	 * 默认构造方法
	 */
	public BaseParse()
	{
		parseType = JSON;
	}
	
	/**
	 * @param type
	 * 具有解析类型的构造方法
	 */
	public BaseParse(int type)
	{
		parseType=type;
	}
	
	/**
	 * @param data，网络返回需要解析的数据
	 * @return true表示解析成功，false表示失败
	 * 抽象解析方法，每个具体的解析类会重写该方法
	 */
	public  boolean parse(byte[]data)
	{
		if(null==data)
		{
			return false;
		}
		if(parseType==Binary)
		{
			return true;
		}
		else if(parseType==JSON)
		{
			try
			{
				String tems = new String(data,"utf-8");//GB2312
				String filt =tems.replace("\r\n","");
				filt =filt.replace("\r","");
				filt = filt.replace("\n","");
				jsonObject =new JSONObject(filt);
				if(jsonObject.has("errorcode") && !jsonObject.isNull("errorcode") )
				{
					String kk = jsonObject.getString("errorcode");
					try
					{
						errorCode=Integer.parseInt(kk);
					}
					catch( NumberFormatException ne)
					{
						
					}
					
				}
				if(jsonObject.has("message") && !jsonObject.isNull("message"))
				{
					message=jsonObject.getString("message");
				}
				return true;
			}
			catch (JSONException e)
			{
				e.printStackTrace();
			}
			catch (UnsupportedEncodingException e)
			{
				e.printStackTrace();
			}
		}
		else if(parseType==XML)
		{
			ByteArrayInputStream bis = new ByteArrayInputStream(data);
			doc = BuildXmlFactory.getDom(bis);
			return true;
		}
		
		return false;
	}
}
