package com.andframework.parse;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xmlpull.v1.XmlSerializer;

import android.util.Log;
import android.util.Xml;


/**
 * @author xudeliang
 * xml dom获取接口
 */
public class BuildXmlFactory
{
	
	
	/**
	 * @param array
	 * @return 
	 * 根据数组创建xml文件
	 */
	public static String buildXml(ArrayList<String>array) 
	{
		if(array.size()==0)
		{
			return null;
		}
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = null;
		try
		{
			db = dbf.newDocumentBuilder();
			Document doc=db.newDocument();;
			Element root=doc.createElement(array.get(0));//根节点
			doc.appendChild(root);
			for(int i=1;i<array.size()-1;i++)
			{
				String tag = array.get(i);
				String text = array.get(i+1);
				doc.createElement(tag);
				doc.createTextNode(text);
			}
			XmlSerializer serializer = Xml.newSerializer();
			StringWriter writer = new StringWriter();
			serializer.setOutput(writer);
			serializer.startDocument("UTF-8", true);
			serializer.startTag("", root.getNodeName());
			String text  = root.getNodeValue();
			if(text!=null)
			{
				serializer.text(text);
			}
			NamedNodeMap map = root.getAttributes();
			int attrSize = map.getLength();
			
			for(int i = 0;i<attrSize;i++)
			{
				Node attrNode = map.item(i);
				String name = attrNode.getNodeName();
				String value = attrNode.getNodeValue();
				serializer.attribute("", name, value);
			}
			serializer.endDocument();
			
			return writer.toString();
		}
		catch (ParserConfigurationException pce)
		{
			
			Log.i("buildXml ParserConfigurationException:", pce.getMessage());
		}
		catch (IllegalArgumentException e)
		{
			// TODO Auto-generated catch block
			Log.i("buildXml IllegalArgumentException:", e.getMessage());
		}
		catch (IllegalStateException e)
		{
			// TODO Auto-generated catch block
			Log.i("buildXml IllegalStateException:", e.getMessage());
		}
		catch (IOException e)
		{
			// TODO Auto-generated catch block
			Log.i("buildXml IOException:", e.getMessage());
		}
		
		
		return null;
		
	}
	
	/**
	 * @param in 输入流
	 * @return 返回dom对方
	 */
	public static Document getDom(InputSource in)
	{
		
		Document dom=null;
		DocumentBuilderFactory dbfactory=DocumentBuilderFactory.newInstance();  
		
		try
		{
			DocumentBuilder db=dbfactory.newDocumentBuilder();
			dom=db.parse(in);
		}
		catch (SAXException e)
		{
			// TODO Auto-generated catch block
			Log.i("getDom(InputSource in) SAXException:", e.getMessage());
		}
		catch (IOException e)
		{
			// TODO Auto-generated catch block
			Log.i("getDom(InputSource in) IOException:", e.getMessage());
		}  
		//Element docEle=dom.getDocumentElement();  
		catch (ParserConfigurationException e)
		{
			// TODO Auto-generated catch block
			Log.i("getDom(InputSource in) ParserConfigurationException:", e.getMessage());
		}
		return dom;
		
	}
	
	/**
	 * @param in 文件参数
	 * @return 返回dom对方
	 */
	public static Document getDom(File in)
	{
		
		Document dom=null;
		DocumentBuilderFactory dbfactory=DocumentBuilderFactory.newInstance();  
		
		//解析地震feed  
		try
		{
			DocumentBuilder db=dbfactory.newDocumentBuilder();
			dom=db.parse(in);
		}
		catch (SAXException e)
		{
			// TODO Auto-generated catch block
			Log.i("getDom(File in) SAXException:", e.getMessage());
		}
		catch (IOException e)
		{
			// TODO Auto-generated catch block
			Log.i("getDom(File in) IOException:", e.getMessage());
		}  
		//Element docEle=dom.getDocumentElement();  
		catch (ParserConfigurationException e)
		{
			// TODO Auto-generated catch block
			Log.i("getDom(File in) ParserConfigurationException:", e.getMessage());
		}
		return dom;
		
	}
	/**
	 * @param in 输入流
	 * @return 返回dom对方
	 */
	public static Document getDom(InputStream in)
	{
		
		Document dom=null;
		DocumentBuilderFactory dbfactory=DocumentBuilderFactory.newInstance();  
		
		try
		{
			dbfactory.setValidating(false);
			DocumentBuilder db=dbfactory.newDocumentBuilder();
			dom=db.parse(in);
		}
		catch (SAXException e)
		{
			// TODO Auto-generated catch block
			Log.i("getDom(InputStream in) SAXException:", e.getMessage());
		}
		catch (IOException e)
		{
			// TODO Auto-generated catch block
			Log.i("getDom(InputStream in) IOException:", e.getMessage());
		}  
		//Element docEle=dom.getDocumentElement();  
		catch (ParserConfigurationException e)
		{
			// TODO Auto-generated catch block
			Log.i("getDom(InputStream in) ParserConfigurationException:", e.getMessage());
		}
		return dom;
		
	}
	/**
	 * @param in 输入文本
	 * @return 返回dom对方
	 */
	public static Document getDom(String in)
	{
		
		Document dom=null;
		DocumentBuilderFactory dbfactory=DocumentBuilderFactory.newInstance();  
		
		try
		{
			DocumentBuilder db=dbfactory.newDocumentBuilder();
			dom=db.parse(in);
		}
		catch (SAXException e)
		{
			// TODO Auto-generated catch block
			Log.i("getDom(String in) SAXException:", e.getMessage());
		}
		catch (IOException e)
		{
			// TODO Auto-generated catch block
			Log.i("getDom(String in) IOException:", e.getMessage());
		}  
		//Element docEle=dom.getDocumentElement();  
		catch (ParserConfigurationException e)
		{
			// TODO Auto-generated catch block
			Log.i("getDom(String in) ParserConfigurationException:", e.getMessage());
		}
		return dom;
		
	}
}
