package com.andframework.ui;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedList;

import com.andframework.util.Util;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.util.AttributeSet;
import android.util.Log;
import android.util.TypedValue;
import android.view.Gravity;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.LinearInterpolator;
import android.view.animation.RotateAnimation;
import android.widget.AbsListView;
import android.widget.BaseAdapter;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.AbsListView.OnScrollListener;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;
import android.widget.TextView;

public class CustomListView extends ListView implements OnScrollListener 
{

	private static final String TAG = "listview";

	private final static int RELEASE_To_REFRESH = 0;
	private final static int PULL_To_REFRESH = 1;
	private final static int REFRESHING = 2;
	private final static int DONE = 3;
	private final static int LOADING = 4;
	
	private final static int GETMOREING=5;

	// 实际的padding的距离与界面上偏移距离的比例
	private final static int RATIO = 3;

	//private LayoutInflater inflater;

	private LinearLayout headView;
	private RelativeLayout bottomView;
	
	private TextView tipsTextview;
	private TextView lastUpdatedTextView;
	private ImageView arrowImageView;
	private ProgressBar progressBar;

	//更多lable
	private TextView moreTextview;

	private ProgressBar bottomProgressBar;
	
	private RotateAnimation animation;
	private RotateAnimation reverseAnimation;

	// 用于保证startY的值在一个完整的touch事件中只被记录一次
	private boolean isRecored;

	private int headContentWidth;
	private int headContentHeight;

	private int startY;
	private int firstItemIndex;

	private int state;

	private boolean isBack;

	private OnRefreshListener refreshListener;
	private OnGetMoreListener getMoreListener;
	
	private boolean isRefreshable;
	private ArrayList<Object> mListViews;
	
	private Bitmap bmap;
	
	private TableListAdapter tableListAdapter;
	
	//默认20条
	private int pageSize=20;
	
	
	//private GetViewAdapter getViewAdapter;
	public CustomListView(Context context) 
	{
		super(context);
		init(context);
	}

	public CustomListView(Context context, AttributeSet attrs)
	{
		super(context, attrs);
		init(context);
	}

	//设置页面条数
	public void setPageSize(int pageSize)
	{
		this.pageSize=pageSize;
	}
	
	
	//返回视图列表
	public ArrayList<Object> getRowViewList()
	{
		return mListViews;
	}
	
	//初始化底部界面
	private void initBottom(Context context)
	{
		bottomView = new RelativeLayout(context);
		ListView.LayoutParams llp =new ListView.LayoutParams(ListView.LayoutParams.FILL_PARENT,Util.dip2px(context, 40));
		bottomView.setLayoutParams(llp);
		
		RelativeLayout.LayoutParams	rlay = new RelativeLayout.LayoutParams(RelativeLayout.LayoutParams.WRAP_CONTENT,RelativeLayout.LayoutParams.WRAP_CONTENT);
		
		moreTextview =new TextView(context);
		moreTextview.setId(12321);
		moreTextview.setText("更多");
		moreTextview.setTextSize(TypedValue.COMPLEX_UNIT_DIP, 20);
		moreTextview.setGravity(Gravity.CENTER);
		
		//rlay.gravity = Gravity.CENTER;
		
		rlay.addRule(RelativeLayout.CENTER_IN_PARENT);
		bottomView.addView(moreTextview, rlay);
		bottomView.setOnTouchListener(new OnTouchListener()
		{
			
			@Override
			public boolean onTouch(View v, MotionEvent event)
			{
				// TODO Auto-generated method stub
				if(event.getAction()==MotionEvent.ACTION_UP)
				{
					if(state != DONE)
					{
						return true ;
					}
					state = GETMOREING;
					changeBottomViewByState();
					onGetMoreData();
					return true;
				}
				return true;
			}
	    });
		
		//rlay.gravity = Gravity.CENTER;
		rlay = new RelativeLayout.LayoutParams(RelativeLayout.LayoutParams.WRAP_CONTENT,RelativeLayout.LayoutParams.WRAP_CONTENT);
		bottomProgressBar = new ProgressBar(context,null,android.R.attr.progressBarStyleSmall);
		bottomProgressBar.setIndeterminate(true);
		bottomProgressBar.setVisibility(View.GONE);
		rlay.addRule(RelativeLayout.LEFT_OF, 12321);
		rlay.addRule(RelativeLayout.CENTER_VERTICAL);
		bottomView.addView(bottomProgressBar,rlay);
	
		
	}
	
	//初始化头部界面
	private void initHeader(Context context)
	{
		headView = new LinearLayout(context);
		ListView.LayoutParams llp =new ListView.LayoutParams(ListView.LayoutParams.FILL_PARENT,ListView.LayoutParams.WRAP_CONTENT);
		//headView.setPadding(0, Util.dip2px(context, 10), 0, 15);
		//headView.setGravity(Gravity.CENTER);
		headView.setLayoutParams(llp);
		
		RelativeLayout rl = new RelativeLayout(context);
		RelativeLayout.LayoutParams rlay = new RelativeLayout.LayoutParams(RelativeLayout.LayoutParams.FILL_PARENT,RelativeLayout.LayoutParams.WRAP_CONTENT);
		rl.setPadding(Util.dip2px(context, 30), 0, 0, 0);
		//rl.setLayoutParams(rlay);
		headView.addView(rl, rlay);
		
		FrameLayout flt = new FrameLayout(context);
		
		rlay = new RelativeLayout.LayoutParams(RelativeLayout.LayoutParams.WRAP_CONTENT,RelativeLayout.LayoutParams.WRAP_CONTENT);
		
		rlay.addRule(RelativeLayout.ALIGN_PARENT_LEFT);
		rlay.addRule(RelativeLayout.CENTER_VERTICAL);
		
		rl.addView(flt, rlay);
		arrowImageView = new ImageView(context);
		//arrowImageView.setImageResource(R.drawable.arrow);
		
		InputStream is = this.getClass().getResourceAsStream("arrow.png");//这种加载速度慢
		bmap = BitmapFactory.decodeStream(is);
		arrowImageView.setImageBitmap(bmap);
		arrowImageView.setMinimumWidth(70);
		arrowImageView.setMinimumHeight(50);
		FrameLayout.LayoutParams fllp = new FrameLayout.LayoutParams(FrameLayout.LayoutParams.WRAP_CONTENT,FrameLayout.LayoutParams.WRAP_CONTENT);
		flt.addView(arrowImageView, fllp);
		
		
		fllp = new FrameLayout.LayoutParams(FrameLayout.LayoutParams.WRAP_CONTENT,FrameLayout.LayoutParams.WRAP_CONTENT);
		progressBar = new ProgressBar(context,null,android.R.attr.progressBarStyleSmall);
		progressBar.setIndeterminate(true);
		progressBar.setVisibility(View.GONE);
		flt.addView(progressBar, fllp);
		
		LinearLayout textLayout = new LinearLayout(context);
		
		textLayout.setOrientation(LinearLayout.VERTICAL);
		textLayout.setGravity(Gravity.CENTER_HORIZONTAL);
		rlay = new RelativeLayout.LayoutParams(RelativeLayout.LayoutParams.WRAP_CONTENT,RelativeLayout.LayoutParams.WRAP_CONTENT);
		rlay.addRule(RelativeLayout.CENTER_HORIZONTAL);
		rl.addView(textLayout, rlay);
		
		
		tipsTextview = new TextView(context);
		LinearLayout.LayoutParams  tipsly = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT,LinearLayout.LayoutParams.WRAP_CONTENT);
		tipsTextview.setTextColor(Color.WHITE);
		//tipsTextview.setBackgroundColor(Color.RED);
		tipsTextview.setText("下拉刷新");
		tipsTextview.setTextSize(TypedValue.COMPLEX_UNIT_DIP, 20);
		
		textLayout.addView(tipsTextview, tipsly);
		
		lastUpdatedTextView = new TextView(context);
		 tipsly = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT,LinearLayout.LayoutParams.WRAP_CONTENT);
		 lastUpdatedTextView.setTextColor(Color.parseColor("#b89766"));
		
		 lastUpdatedTextView.setText("上次更新");
		 lastUpdatedTextView.setTextSize(TypedValue.COMPLEX_UNIT_DIP, 10);
		
		textLayout.addView(lastUpdatedTextView, tipsly);
		
		
		measureView(headView);
		headContentHeight = headView.getMeasuredHeight();
		headContentWidth = headView.getMeasuredWidth();

		headView.setPadding(0, -1 * headContentHeight, 0, 0);
		headView.invalidate();

		Log.v("size", "width:" + headContentWidth + " height:"
				+ headContentHeight);

		addHeaderView(headView, null, false);
	}
	
	
	
	private void init(Context context) 
	{
		setCacheColorHint(Color.parseColor("#00000000"));//context.getResources().getColor(R.color.transparent));
		
		initHeader(context);
		initBottom(context);
		
		setOnScrollListener(this);

		animation = new RotateAnimation(0, -180,
				RotateAnimation.RELATIVE_TO_SELF, 0.5f,
				RotateAnimation.RELATIVE_TO_SELF, 0.5f);
		animation.setInterpolator(new LinearInterpolator());
		animation.setDuration(250);
		animation.setFillAfter(true);

		reverseAnimation = new RotateAnimation(-180, 0,
				RotateAnimation.RELATIVE_TO_SELF, 0.5f,
				RotateAnimation.RELATIVE_TO_SELF, 0.5f);
		reverseAnimation.setInterpolator(new LinearInterpolator());
		reverseAnimation.setDuration(200);
		reverseAnimation.setFillAfter(true);

		state = DONE;
		isRefreshable = false;
		
		mListViews = new ArrayList<Object>();
		tableListAdapter= new TableListAdapter(context,mListViews);
	    this.setAdapter(tableListAdapter);
	    lastUpdatedTextView.setText("最近更新:" + new Date().toLocaleString());
	}

	
	//设置更多是否可见
	public void setBottomViewVisible(boolean visible)
	{
		if(visible && getFooterViewsCount()<=0)
		{
			addFooterView(bottomView, null, false);
		}
		else if(!visible &&getFooterViewsCount()>0)
		{
			this.removeFooterView(bottomView);//.removeHeaderView(bottomView);
		}
	}
	
	//设置箭头图片
	public void setArrowImage(int resid)
	{
		arrowImageView.setImageResource(resid);
	}
	
	//设置头部背景
	public void setHeadBackgrond(int resid)
	{
		headView.setBackgroundResource(resid);
	}
	
	//设置底部背景
	public void setBottomBackgrond(int resid)
	{
		bottomView.setBackgroundResource(resid);
	}
	
	//设置头部背景
	public void setHeadBackgrondColor(int Color)
	{
		headView.setBackgroundColor(Color);
	}
	
	//设置底部背景
	public void setBottomBackgrondColor(int Color)
	{
		bottomView.setBackgroundColor(Color);
	}
	
	//设置底部文字颜色
	public void setMoreTextColor(int color)
	{
		moreTextview.setTextColor(color);
	}
	
	public void onScroll(AbsListView arg0, int firstVisiableItem, int arg2,
			int arg3) {
		firstItemIndex = firstVisiableItem;
	}
	
	//设置更新文本颜色
	public void setUpdateTextColor(int color)
	{
		tipsTextview.setTextColor(color);
	}
	
	//设置时间文本颜色
	public void setTimeTextColor(int color)
	{
		lastUpdatedTextView.setTextColor(color);
	}
	
	public void onScrollStateChanged(AbsListView arg0, int arg1) {
	}

	public boolean onTouchEvent(MotionEvent event) 
	{

		if (isRefreshable) {
			switch (event.getAction()) {
			case MotionEvent.ACTION_DOWN:
				if (firstItemIndex == 0 && !isRecored) {
					isRecored = true;
					startY = (int) event.getY();
					Log.v(TAG, "在down时候记录当前位置‘");
				}
				break;

			case MotionEvent.ACTION_UP:

				if (state != REFRESHING && state != LOADING) {
					if (state == DONE) {
						// 什么都不做
					}
					if (state == PULL_To_REFRESH) {
						state = DONE;
						changeHeaderViewByState();

						Log.v(TAG, "由下拉刷新状态，到done状态");
					}
					if (state == RELEASE_To_REFRESH) {
						state = REFRESHING;
						changeHeaderViewByState();
						onRefresh();

						Log.v(TAG, "由松开刷新状态，到done状态");
					}
				}

				isRecored = false;
				isBack = false;

				break;

			case MotionEvent.ACTION_MOVE:
				int tempY = (int) event.getY();

				if (!isRecored && firstItemIndex == 0) {
					Log.v(TAG, "在move时候记录下位置");
					isRecored = true;
					startY = tempY;
				}

				if (state != REFRESHING && isRecored && state != LOADING) {

					// 保证在设置padding的过程中，当前的位置一直是在head，否则如果当列表超出屏幕的话，当在上推的时候，列表会同时进行滚动

					// 可以松手去刷新了
					if (state == RELEASE_To_REFRESH) {

						setSelection(0);

						// 往上推了，推到了屏幕足够掩盖head的程度，但是还没有推到全部掩盖的地步
						if (((tempY - startY) / RATIO < headContentHeight)
								&& (tempY - startY) > 0) {
							state = PULL_To_REFRESH;
							changeHeaderViewByState();

							Log.v(TAG, "由松开刷新状态转变到下拉刷新状态");
						}
						// 一下子推到顶了
						else if (tempY - startY <= 0) {
							state = DONE;
							changeHeaderViewByState();

							Log.v(TAG, "由松开刷新状态转变到done状态");
						}
						// 往下拉了，或者还没有上推到屏幕顶部掩盖head的地步
						else {
							// 不用进行特别的操作，只用更新paddingTop的值就行了
						}
					}
					// 还没有到达显示松开刷新的时候,DONE或者是PULL_To_REFRESH状态
					if (state == PULL_To_REFRESH) {

						setSelection(0);

						// 下拉到可以进入RELEASE_TO_REFRESH的状态
						if ((tempY - startY) / RATIO >= headContentHeight) {
							state = RELEASE_To_REFRESH;
							isBack = true;
							changeHeaderViewByState();

							Log.v(TAG, "由done或者下拉刷新状态转变到松开刷新");
						}
						// 上推到顶了
						else if (tempY - startY <= 0) {
							state = DONE;
							changeHeaderViewByState();

							Log.v(TAG, "由DOne或者下拉刷新状态转变到done状态");
						}
					}

					// done状态下
					if (state == DONE) {
						if (tempY - startY > 0) {
							state = PULL_To_REFRESH;
							changeHeaderViewByState();
						}
					}

					// 更新headView的size
					if (state == PULL_To_REFRESH) {
						headView.setPadding(0, -1 * headContentHeight
								+ (tempY - startY) / RATIO, 0, 0);

					}

					// 更新headView的paddingTop
					if (state == RELEASE_To_REFRESH) {
						headView.setPadding(0, (tempY - startY) / RATIO
								- headContentHeight, 0, 0);
					}

				}

				break;
			}
		}

		return super.onTouchEvent(event);
	}

	
	// 当状态改变时候，调用该方法，以更新界面 ， 更改底部的View视图状态
	private void changeBottomViewByState() 
	{
		switch (state) {
		case GETMOREING:
			bottomProgressBar.setVisibility(View.VISIBLE);
			moreTextview.setText("正在获取...");
			Log.v(TAG, "当前状态,更多 正在获取...");
			break;
		case DONE:
			bottomProgressBar.setVisibility(View.GONE);
			moreTextview.setText("更多");
			Log.v(TAG, "当前状态，更多 done");
			break;
		}
	}
	
	// 当状态改变时候，调用该方法，以更新界面
	private void changeHeaderViewByState() 
	{
		switch (state) {
		case RELEASE_To_REFRESH:
			arrowImageView.setVisibility(View.VISIBLE);
			progressBar.setVisibility(View.GONE);
			tipsTextview.setVisibility(View.VISIBLE);
			lastUpdatedTextView.setVisibility(View.VISIBLE);

			arrowImageView.clearAnimation();
			arrowImageView.startAnimation(animation);

			tipsTextview.setText("松开刷新");

			Log.v(TAG, "当前状态，松开刷新");
			break;
		case PULL_To_REFRESH:
			progressBar.setVisibility(View.GONE);
			tipsTextview.setVisibility(View.VISIBLE);
			lastUpdatedTextView.setVisibility(View.VISIBLE);
			arrowImageView.clearAnimation();
			arrowImageView.setVisibility(View.VISIBLE);
			// 是由RELEASE_To_REFRESH状态转变来的
			if (isBack) {
				isBack = false;
				arrowImageView.clearAnimation();
				arrowImageView.startAnimation(reverseAnimation);

				tipsTextview.setText("下拉刷新");
			} else {
				tipsTextview.setText("下拉刷新");
			}
			Log.v(TAG, "当前状态，下拉刷新");
			break;

		case REFRESHING:

			headView.setPadding(0, 0, 0, 0);

			progressBar.setVisibility(View.VISIBLE);
			arrowImageView.clearAnimation();
			arrowImageView.setVisibility(View.GONE);
			tipsTextview.setText("正在刷新...");
			lastUpdatedTextView.setVisibility(View.VISIBLE);

			Log.v(TAG, "当前状态,正在刷新...");
			break;
		case DONE:
			headView.setPadding(0, -1 * headContentHeight, 0, 0);

			progressBar.setVisibility(View.GONE);
			arrowImageView.clearAnimation();
			//arrowImageView.setImageResource(R.drawable.arrow);
			arrowImageView.setImageBitmap(bmap);
			tipsTextview.setText("下拉刷新");
			lastUpdatedTextView.setVisibility(View.VISIBLE);

			Log.v(TAG, "当前状态，done");
			break;
		}
	}

	//设置刷新监听
	public void setonRefreshListener(OnRefreshListener refreshListener) 
	{
		this.refreshListener = refreshListener;
		isRefreshable = true;
	}

	//设置获取更多监听
	public void setonGetMoreListener(OnGetMoreListener getMoreListener)
	{
		this.getMoreListener = getMoreListener;
		//isGetMoreable = true;
	}
	
	
	public void addAllView(LinkedList<Object> all)
	{
		mListViews.addAll(all);
		tableListAdapter.notifyDataSetChanged();
    }
	
    public void addViewToFirst(Object v)
    {
    	mListViews.add(0, v);
    	tableListAdapter.notifyDataSetChanged();
    }
    
    public void addViewToLast(Object v)
    {
    	mListViews.add(v);
    	tableListAdapter.notifyDataSetChanged();
    }
    public void addViewToLast(Object v,boolean fresh)
    {
    	mListViews.add(v);
    	if(fresh)
    	tableListAdapter.notifyDataSetChanged();
    }
    public void addViewToIndex(int index,Object v)
    {
    	mListViews.add(index, v);
    	tableListAdapter.notifyDataSetChanged();
    }
    
    public void removeAllView()
    {
    	mListViews.clear();
    	setBottomViewVisible(false);
    	tableListAdapter.notifyDataSetChanged();
    }
    
    
    public void removeView(int index)
    {

    	if(mListViews.size()>0)
    	{
    		mListViews.remove(index);
        	tableListAdapter.notifyDataSetChanged();
        	if(mListViews.size()==0)
        	{
        		setBottomViewVisible(false);
        	}
    	}
    	
    	
    }
    
    public void removeView(Object view)
    {
    	if(mListViews.size()>0)
    	{
    		mListViews.remove(view);//.remove(index);
        	tableListAdapter.notifyDataSetChanged();
        	if(mListViews.size()==0)
        	{
        		setBottomViewVisible(false);
        	}
    	}
    	
    }
    
    
    //刷新接口
	public interface OnRefreshListener 
	{
		public void onRefresh();
	}
	
	//获取更多监听接口
	public interface OnGetMoreListener 
	{
		public void onGetmore();
	}
	
	//还原状态
	public void retsetStatus()
	{
		state = DONE;
		changeBottomViewByState();
		tableListAdapter.notifyDataSetChanged();
		setBottomViewVisible(false);
	}
	
	//完成刷新调用，外部使用
	public void onRefreshComplete() 
	{
		state = DONE;
		lastUpdatedTextView.setText("最近更新:" + new Date().toLocaleString());
		changeHeaderViewByState();
		// If refresh view is visible when loading completes, scroll down to
        // the next item.
        if (headView.getBottom() > 0) 
        {
            invalidateViews();
            setSelection(1);
        }
        tableListAdapter.notifyDataSetChanged();
        
        //设置更多是否可见
        if(null!=mListViews &&mListViews.size()>0)
        {
        	
        	int tag = mListViews.size()%pageSize;
        	if(0==tag)
        	{
        		setBottomViewVisible(true);
        	}
        	else
        	{
        		setBottomViewVisible(false);
        	}
        }
	}

	//获取更多数据完成，外部使用
	public void onGetMoreComplete() 
	{
		invalidateViews();
		state = DONE;
		changeBottomViewByState();
		if(null!=mListViews &&mListViews.size()>0)
		{
			setScrollToRow(mListViews.size()-pageSize);
		}
		
		tableListAdapter.notifyDataSetChanged();
		
		 //设置更多是否可见
		if(null!=mListViews &&mListViews.size()>0)
        {
        	
        	int tag = mListViews.size()%pageSize;
        	if(0==tag)
        	{
        		setBottomViewVisible(true);
        	}
        	else
        	{
        		setBottomViewVisible(false);
        	}
        }
	}
	
	/**
	 * 
	 * 
	 * @param row 定位显示的行
	 * @return
	 * 
	 */
	public void setScrollToRow(int row)
	{
		this.setSelection(row);
	}
	
	//方便外面调用刷新接口，可以不用手动滑动列表
	public void freshData()
	{
		state = REFRESHING;
		changeHeaderViewByState();
		onRefresh();
	}
	
	
	//调用刷新接口
	private void onRefresh() 
	{
		if (refreshListener != null) {
			refreshListener.onRefresh();
		}
	}
	
	//调用获取更多数据接口
	private void onGetMoreData()
	{
		if (getMoreListener != null) 
		{
			getMoreListener.onGetmore();
		}
	}
	

	// 调整child的width以及height
	private void measureView(View child) 
	{
		ViewGroup.LayoutParams p = child.getLayoutParams();
		if (p == null) {
			p = new ViewGroup.LayoutParams(ViewGroup.LayoutParams.FILL_PARENT,
					ViewGroup.LayoutParams.WRAP_CONTENT);
		}
		int childWidthSpec = ViewGroup.getChildMeasureSpec(0, 0 + 0, p.width);
		int lpHeight = p.height;
		int childHeightSpec;
		if (lpHeight > 0) {
			childHeightSpec = MeasureSpec.makeMeasureSpec(lpHeight,
					MeasureSpec.EXACTLY);
		} else {
			childHeightSpec = MeasureSpec.makeMeasureSpec(0,
					MeasureSpec.UNSPECIFIED);
		}
		child.measure(childWidthSpec, childHeightSpec);
	}

	public int getListNum()
	{
		if(mListViews!=null)
		{
			return  mListViews.size();
		}
		
		return 0;
	}

	public void setGetViewAdapter(GetViewAdapter getviewadapter)
	{
		tableListAdapter.setGetViewAdapter(getviewadapter);
	}
	//列表适配器
    private class TableListAdapter extends BaseAdapter 
    {
    	private Context mContext;
    	private ArrayList<Object> views;
    	private GetViewAdapter getviewadapter;
    	
    	public void setGetViewAdapter(GetViewAdapter getviewadapter)
    	{
    		this.getviewadapter=getviewadapter;
    	}
    	
        public TableListAdapter(Context context,ArrayList<Object> v) 
        {
            mContext = context;
            views = v;
            
            
        }
        @Override  
        public int getItemViewType(int position) {  
            
        	if(getviewadapter!=null)
        	{
        		return getviewadapter.getItemViewType(position);
        	}
        	else 
        		
            return 0;
             
             
       }  
       @Override  
       public int getViewTypeCount() {  
    	   if(getviewadapter!=null)
       	{
       		return getviewadapter.getViewTypeCount();
       	}
       	else 
           return 1;  
     
       } 
       @Override  
        public int getCount() 
        {
            return views.size();
        }
       @Override  
        public Object getItem(int position) 
        {
            return views.get(position);//.getTag();
        }
        
        @Override
		public long getItemId(int position)
		{
			return position;
		}

        public View getView(int position, View convertView, ViewGroup parent)
        {
        	Object obj =views.get(position);
        	if(getviewadapter!=null)
        	{
        		return getviewadapter.getView( position,  convertView,  parent,obj);
        	}
            return convertView;
        }
    }
}

