/**
 * 
 */
package com.andframework.ui;

import com.andframework.R;
import com.andframework.config.LocalData;
import com.andframework.parse.VersionParse;
import com.andframework.util.Util;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.app.AlertDialog.Builder;
import android.content.Context;
import android.content.DialogInterface;
import android.view.Gravity;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

/**
 * @author xudeliang
 *
 */
public class CustomMessageShow
{
	private  ProgressDialog progressDialog ;
	
	
	private static CustomMessageShow inst;
	
	private CustomMessageShow()
	{
		
	}
	
	public static CustomMessageShow getInst()
	{
		if(null==inst)
		{
			inst=new CustomMessageShow();
		}
		
		return inst;
	}
	
	/**
	 * @param context 上下文
	 * 显示默认的提示框
	 */
	public void showProgressDialog(Activity context)
	{
		if(context.isFinishing())return;
		cancleProgressDialog();
		progressDialog = new ProgressDialog(context);
		progressDialog.setMessage("正在加载数据！");
		progressDialog.setIndeterminate(true);
		progressDialog.setCancelable(true);
		progressDialog.show();
	}
	
	/**
	 *
	 * 取消显示 
	 */
	public void cancleProgressDialog()
	{
		if (null != progressDialog &&progressDialog.isShowing())
		{
			progressDialog.dismiss();
		}
		progressDialog=null;
	}
	
	/**
	 * @param context 上下文
	 * @param message 显示的文本
	 */
	public void showProgressDialog(Activity context,String message)
	{
		if(context.isFinishing())return;
		
		if (null != progressDialog&& progressDialog.isShowing())
		{
			progressDialog.dismiss();
		}
		progressDialog = new ProgressDialog(context);
		progressDialog.setMessage(message);
		progressDialog.setIndeterminate(true);
		progressDialog.setCancelable(true);
		progressDialog.show();
	}
	
	
	/**
	 * @param context 上下文
	 * @param message 显示的文本
	 */
	public void showShortToast(Activity context,String message)
	{
		if(!context.isFinishing())
		Toast.makeText(context,message, Toast.LENGTH_SHORT).show();
	}
	
	
	/**
	 * @param context 上下文
	 * @param message 显示的文本
	 */
	public void showLongToast(Activity context,String message)
	{
		if(!context.isFinishing())
		Toast.makeText(context,message, Toast.LENGTH_LONG).show();
	}
	
	//深度定制对话框 String left,String middle,String right 对应PositiveButton，NeutralButton NegativeButton
	public AlertDialog showConfirmDialog(Activity context,String message,String title,String left,String middle,String right,int iconid,DialogInterface.OnClickListener l)
	{
		Builder builder = new AlertDialog.Builder(context);
		if(iconid>0)
		builder.setIcon(iconid);
		builder.setTitle("提示信息");
		
		if(title!=null)
		builder.setTitle(title);
		
		String leftStr = "确认";
		if(left!=null)
			leftStr = left;
		
		String rightStr = "取消";
		if(right!=null)
			rightStr = right;
		
		
		builder.setMessage(message);
		builder.setPositiveButton(leftStr, l);//
		
		
		if(null!=middle && middle.length()>0)
		{
			builder.setNeutralButton(middle, l);
		}
		
		builder.setNegativeButton (rightStr, l);
		
		AlertDialog dialog = builder.create();
		dialog.show();
		return dialog;
	}
	
	/**
	 * @param context 确认提醒
	 * @param message 提示文本
	 * @param 操作 监听
	 */
	
	public void showConfirmDialog(Activity context,String message,DialogInterface.OnClickListener l)
	{
		Builder builder = new AlertDialog.Builder(context);
		builder.setTitle("提示信息");
		String leftStr = "确认";
		String rightStr = "取消";
		builder.setMessage(message);
		builder.setPositiveButton(leftStr, l);//
		builder.setNeutralButton(rightStr, l);
		AlertDialog dialog = builder.create();
		dialog.show();
	}
	
	/**
	 * @param context 退出提醒
	 * @param message 显示的文本
	 */
	
	public void showExitDialog(Context context,String message)
	{
		Builder builder = new AlertDialog.Builder(context);
		builder.setTitle("提示信息");
		String text = message;
		String leftStr = "确认";
		String rightStr = "取消";
		builder.setMessage(text);
		builder.setPositiveButton(leftStr,
				new DialogInterface.OnClickListener()
				{
					public void onClick(DialogInterface d, int i)
					{
						//context.finish();
						android.os.Process.killProcess(android.os.Process.myPid());
						System.exit(1);
					}
				});
		
		builder.setNeutralButton(rightStr,
				new DialogInterface.OnClickListener()
				{
					public void onClick(DialogInterface d, int i)
					{
						
					}
				});
		AlertDialog dialog = builder.create();
		dialog.show();
	}
	
	/**
	 * @param context 
	 * @param versionParse 
	 */
	
	public void showUpdateDialog(Activity context,VersionParse versionParse)
	{
		if(versionParse.versionInfo.updateflag.equals("2"))
		{
			showShortToast(context,"本版本是最新版本！");
			return;
		}
		
		
		
		Builder builder = new AlertDialog.Builder(context);
		builder.setTitle("发现新版本，是否要升级？");
		
		View view = buildUpdatePanel(context,versionParse);
		builder.setView(view);
		AlertDialog dialog = builder.create();
	
		if(versionParse.versionInfo.updateflag.equals("1"))//强制升级不能返回
		{
			dialog.setCancelable(false);
		}
		view.setTag(dialog);
		
		dialog.show();
		
		
	}
	
	/*
	 *1、 强制升级 不显示不再提醒
	 * 2、强制升级，要不升级，要不退出
	 * 3、非强制升级显示 再提醒，显示升级，下次再说
	 */
	private View buildUpdatePanel(final Activity context,final VersionParse versionParse)
	{
		final LinearLayout view = new LinearLayout(context);
		view.setOrientation(LinearLayout.VERTICAL);
		TextView nameAndSize= new TextView(context);
		nameAndSize.setText("名称："+versionParse.versionInfo.appname+" 文件大小："+versionParse.versionInfo.packsize);
		LinearLayout.LayoutParams lls = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT, LinearLayout.LayoutParams.WRAP_CONTENT);
		lls.leftMargin=Util.dip2px(context, 10);
		lls.rightMargin=Util.dip2px(context, 10);
		view.addView(nameAndSize,lls);
		
		TextView updatetv= new TextView(context);
		updatetv.setText("本次升级：");
		view.addView(updatetv,lls);
		
		updatetv= new TextView(context);
		updatetv.setText(versionParse.versionInfo.des);
		view.addView(updatetv,lls);
		if(versionParse.versionInfo.updateflag.equals("1"))//强制升级
		{
			LinearLayout butons = new LinearLayout(context);
			butons.setOrientation(LinearLayout.HORIZONTAL);
			Button update = new Button(context);
			update.setText("强制升级");
			
			update.setOnClickListener(new OnClickListener(){

				@Override
				public void onClick(View v)
				{
					Util.openBrowser(context, versionParse.versionInfo.packurl);
				}});
			
			Button exit = new Button(context);
			exit.setText("退出程序");
			butons.addView(update);
			butons.addView(exit);
			LinearLayout.LayoutParams ll = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT, LinearLayout.LayoutParams.WRAP_CONTENT);
			ll.gravity = Gravity.CENTER;
			view.addView(butons,ll);
			
			exit.setOnClickListener(new OnClickListener(){

				@Override
				public void onClick(View v)
				{
					android.os.Process.killProcess(android.os.Process.myPid());
				}});
		}
		else //一般升级
		{
			final CheckBox cv = new CheckBox(context);
			cv.setText("不再提醒");
			LinearLayout.LayoutParams ll = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT, LinearLayout.LayoutParams.WRAP_CONTENT);
			ll.gravity = Gravity.CENTER;
			view.addView(cv,ll);
			
			LinearLayout butons = new LinearLayout(context);
			butons.setOrientation(LinearLayout.HORIZONTAL);
			Button update = new Button(context);
			update.setText("版本升级");
			update.setOnClickListener(new OnClickListener(){

				@Override
				public void onClick(View v)
				{
					Util.openBrowser(context, versionParse.versionInfo.packurl);
					((AlertDialog)view.getTag()).cancel();
				}});
			Button nexttime = new Button(context);
			nexttime.setText("下次再说");
			nexttime.setOnClickListener(new OnClickListener(){

				@Override
				public void onClick(View v)
				{
					if(cv.isChecked())
					{
						LocalData.getInst().setLocalValue(LocalData.localDataKey_updatetips, "1");
					}
					else
					{
						LocalData.getInst().setLocalValue(LocalData.localDataKey_updatetips, "0");
					}
					((AlertDialog)view.getTag()).cancel();
				}});
			butons.addView(update);
			butons.addView(nexttime);
			ll = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT, LinearLayout.LayoutParams.WRAP_CONTENT);
			ll.gravity = Gravity.CENTER;
			view.addView(butons,ll);
		}
		return view;
	}
}