package com.andframework.util;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.w3c.dom.Node;

import com.andframework.config.FrameworkConfig;
import com.andframework.zmfile.ZMFile;
import com.andframework.zmfile.ZMFilePath;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.graphics.Rect;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.net.Uri;
import android.telephony.CellLocation;
import android.telephony.NeighboringCellInfo;
import android.telephony.TelephonyManager;
import android.telephony.cdma.CdmaCellLocation;
import android.telephony.gsm.GsmCellLocation;
import android.util.Log;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.view.inputmethod.InputMethodManager;

/**
 * @author 徐德良
 * 工具类
 *
 */
public class Util 
{

	/**
	 * 三种日志类型对应printLog方法
	 */
	public static final int FILELOG = 0;
	public static final int NETLOG = 1;
	public static final int SYSLOG = 2;
	
	
	/**
	 * @param tag 标签类型
	 * @param msg 打印内容
	 * @param type type=0表示文件，type=1表示网络，type=2表示log
	 * @param logLevel Log.DEBUG,ERROR,INFOR,WARM,VERBOS等
	 */
	public static synchronized void  printLog(String tag, String msg, int type, int logLevel) 
	{
		if (type == SYSLOG)
		{
			switch (logLevel) 
			{
			case Log.VERBOSE: 
			{
				Log.v(tag, msg);
				break;
			}
			case Log.DEBUG: 
			{
				Log.d(tag, msg);
				break;
			}
			case Log.INFO: 
			{
				Log.i(tag, msg);
				break;
			}
			case Log.WARN: 
			{
				Log.w(tag, msg);
				break;
			}
			case Log.ERROR: 
			{
				Log.e(tag, msg);
				break;
			}
			}
		} 
		else if (type == FILELOG) 
		{
			String fileFullPath =FrameworkConfig.getInst().getLogPath() + FrameworkConfig.getInst().getLogName();
			try 
			{
				byte[] data = (Util.getFormatTime() + ":" + tag + "->" + msg + "\r\n").getBytes("utf-8");
				File file = new File(fileFullPath);
				if (file.exists()) 
				{
					int pos = ZMFile.getSize(fileFullPath);
					ZMFile.write(fileFullPath, data, 0, data.length, pos);
				} 
				else
				{
					ZMFile.write(fileFullPath, data, 0, data.length, 0);
				}
			} 
			catch (IOException e) 
			{
				e.printStackTrace();
			}
		}

	}
	
	/**
	 * 获取格式化的当前时间，精确到毫秒
	 */
	public static String getFormatTime()
	{
		int tem =0;
		long timestamp =System.currentTimeMillis();
		StringBuffer sb = new StringBuffer();
		Calendar cl = Calendar.getInstance();
		cl.setTime(new Date(timestamp));
		tem = cl.get(Calendar.YEAR);
		sb.append(tem);
		sb.append(":");
		tem = cl.get(Calendar.MONTH)+1;
		sb.append(tem);
		sb.append(":");
		tem = cl.get(Calendar.DAY_OF_MONTH);
		sb.append(tem);
		sb.append(":");
		tem = cl.get(Calendar.HOUR_OF_DAY);
		if(tem<10)
		{
			sb.append("0");
		}
		sb.append(tem);
		sb.append(":");
		tem = cl.get(Calendar.MINUTE);
		if(tem<10)
		{
			sb.append("0");
		}
		sb.append(tem);
		sb.append(":");
		tem = cl.get(Calendar.SECOND);
		if(tem<10)
		{
			sb.append("0");
		}
		sb.append(tem);
		sb.append(":");
		tem = cl.get(Calendar.MILLISECOND);
		if(tem<10)
		{
			sb.append("0");
		}
		sb.append(tem);
		return sb.toString();
	}
	
	public static String getNodeValue(Node node)
	{
		try
		{
			return node.getFirstChild().getNodeValue();
		}
		catch(Exception e)
		{
			
		}
		return null;
	}
	
	/**
	 * 根据手机的分辨率从 dp 的单位 转成为 px(像素)
	 */
	public static int dip2px(Context context, float dpValue) 
	{
		final float scale = context.getResources().getDisplayMetrics().density;
		return (int) (dpValue * scale + 0.5f);
	}
	 
	/**
	 * 根据手机的分辨率从 px(像素) 的单位 转成为 dp
	 */
	public static int px2dip(Context context, float pxValue)
	{
		final float scale = context.getResources().getDisplayMetrics().density;
		return (int) (pxValue / scale + 0.5f);
	}
	
	public static String urlToFileName(String neturl)
	{
		StringBuffer sb = new StringBuffer();
		String url = new String(neturl);
		int index = 0;
		int lastindex =0;
		int len = url.length();
		while(index<len)
		{
			if(url.charAt(index)=='/')
			{
				if(index-lastindex>1&&0!=lastindex)
				{
					String split = url.substring(lastindex+1, index);
					int mh = split.indexOf(':');
					if(mh>0)
					{
						split = split.substring(0, mh);
					}
					sb.append(split);
					sb.append(ZMFilePath.FILE_SEPARATOR);
				}
				lastindex=index;
			}
			index++;
		}
		sb.append(url.substring(lastindex+1, len));
		
		String tem=sb.toString().replace('/', '^');
		tem=tem.replace('?', '^');
		/*pic=picpath+pic;
		int index = pic.lastIndexOf('.');
		pic = pic.substring(0, index);
		pic=pic+".zmpng";*/
		
		return tem;
	}
	
	//邮箱校验
	public static boolean emailValidate(String email)
	{
        Pattern p = Pattern.compile("\\w+([-+.]\\w+)*@\\w+([-.]\\w+)*\\.\\w+([-.]\\w+)*");
        Matcher m = p.matcher(email);
        return m.find();
    }

	
	//隐藏软键盘
	public static void hideInputMethod(View view,Activity act)
	{
		InputMethodManager imm = (InputMethodManager)act.getSystemService(Context.INPUT_METHOD_SERVICE); 
		imm.hideSoftInputFromWindow(view.getWindowToken(), 0);
	}
	
	//打开浏览器
	public static void openBrowser(Context context,String url)
	{
		Uri uri = Uri.parse(url);
		Intent it = new Intent(Intent.ACTION_VIEW, uri);
		context.startActivity(it);
	}
	
	/**
	 * 获取当前客户端的版本号（记录在AndroidManifest.xml的android:versionCode中）
	 */
	public static int getVersionCode(Context context) 
	{
		int version = 0;
		try 
		{
			PackageManager manager = context.getPackageManager();
			PackageInfo info = manager.getPackageInfo(context.getPackageName(), 0);
			version = info.versionCode;
		} 
		catch (NameNotFoundException err) 
		{
			Log.e("getVersionCode", err.getMessage(), err);
		}
		return version;
	}

	
	/**
	 * 获取当前客户端的版本号（记录在AndroidManifest.xml的android:versionCode中）
	 */
	public static String getVersionCodeName(Context context) 
	{
		String version = "";
		try 
		{
			PackageManager manager = context.getPackageManager();
			PackageInfo info = manager.getPackageInfo(context.getPackageName(), 0);
			version = info.versionName;
		} 
		catch (NameNotFoundException err) 
		{
			Log.e("getVersionCodeName", err.getMessage(), err);
		}
		return version;
	}
	
	/**
	 * 获取状态栏高度
	 */
	public static int getStatusBarHeight(Activity activity)
	{
		Rect rect = new Rect();
		Window win = activity.getWindow();
		win.getDecorView().getWindowVisibleDisplayFrame(rect);
		return rect.top;
	}
	
	/**
	 * 设置全屏
	 */
	public static void setFullScreen(Activity activity)
	{
		 //设置无标题  
		activity.requestWindowFeature(Window.FEATURE_NO_TITLE);  
	    //设置全屏  
		activity.getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN,   
	            WindowManager.LayoutParams.FLAG_FULLSCREEN); 
	}
	
	
	/**
	 * locationListener 需要监听位置变化的时候使用，少用耗电
	 */
	public static Location getLocation(Activity act,LocationListener locationListener)
	{
		LocationManager lm = (LocationManager) act.getSystemService(Context.LOCATION_SERVICE);   
		//1 通过NETWORK 获取
		Location location=null;
		try
		{
			location = lm.getLastKnownLocation(LocationManager.NETWORK_PROVIDER);     
			if(location!=null )
			{
				Log.i("getLocation", "from NETWORK");
				return location;
			}
		}
		
		catch(IllegalArgumentException s)
		{
			Log.i("getLocation", "from NETWORK IllegalArgumentException");
		}
		catch(SecurityException s)
		{
			Log.i("getLocation", "from NETWORK SecurityException");
		}
		
		
		//2通过
		Criteria criteria = new Criteria();   
		criteria.setAccuracy(Criteria.ACCURACY_FINE);   
		criteria.setAltitudeRequired(false);   
		criteria.setBearingRequired(false);   
		criteria.setCostAllowed(true);   
		criteria.setPowerRequirement(Criteria.POWER_LOW); 
		String provider = lm.getBestProvider(criteria, true);   
		if(null!=provider)
		{
			try
			{
				location = lm.getLastKnownLocation(provider);   
				if(location!=null )
				{
					Log.i("getLocation", "from Criteria");
					return location;
				}
			}
			catch(IllegalArgumentException s)
			{
				Log.i("getLocation", "from NETWORK 2 IllegalArgumentException");
			}
			catch(SecurityException s)
			{
				Log.i("getLocation", "from NETWORK 2 SecurityException");
			}
			
		}
		
		
		//3通过gps
		try
		{
			location = lm.getLastKnownLocation(LocationManager.GPS_PROVIDER);
			if(location!=null )
			{
				Log.i("getLocation", "from GPS_PROVIDER");
				return location;
			}
		}
		catch(IllegalArgumentException s)
		{
			Log.i("getLocation", "from NETWORK 3 IllegalArgumentException");
		}
		catch(SecurityException s)
		{
			Log.i("getLocation", "from NETWORK 3 SecurityException");
		}
		
		
		
		if(null!=locationListener)
		lm.requestLocationUpdates(LocationManager.NETWORK_PROVIDER,1000, 0,locationListener);     
       
		return location;
	}
	

	public static void shareContent(Context context,String share,String title)
	{
			//实例化一个Intent对象，并且设置Intent的Action为ACTION_SEND
			Intent intent=new Intent(Intent.ACTION_SEND); 
			//设置MIME数据类型
			intent.setType("text/plain"); 
			//设置主题
			intent.putExtra(Intent.EXTRA_SUBJECT, title); 
			//设置内容
			intent.putExtra(Intent.EXTRA_TEXT, share); 
			//启动Activity，并设置菜单标题
			context.startActivity(Intent.createChooser(intent, title)); 
		
	}
	
	//从resources中的raw 文件夹中获取文件并读取数据  
    public static  byte[] getFromRaw(Context context,int RRawId)
    {  
		byte[] result = null;
		try
		{
			InputStream in = context.getResources().openRawResource(RRawId);
			// 获取文件的字节数
			int lenght = in.available();
			// 创建byte数组
			byte[] buffer = new byte[lenght];
			// 将文件中的数据读到byte数组中
			in.read(buffer);
			return buffer;
			// result = EncodingUtils.getString(buffer, ENCODING);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		return result;
    }  
      
    //从assets 文件夹中获取文件并读取数据  
    public static byte[]  getFromAssets(Context context,String fileName)
    {  
		byte[] result = null;
		try
		{
			InputStream in = context.getResources().getAssets().open(fileName);
			// 获取文件的字节数
			int lenght = in.available();
			// 创建byte数组
			byte[] buffer = new byte[lenght];
			// 将文件中的数据读到byte数组中
			in.read(buffer);
			return buffer;
			// result = EncodingUtils.getString(buffer, ENCODING);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		return result;  
    }
}
