package com.andframework.zmfile;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.RandomAccessFile;

import android.os.Environment;
import android.os.StatFs;


/**
 * @author 徐德良
 * 文件操作类
 * 多线程读写的时候需要注意，外部的方法需要同步
 *
 */
public  final class  ZMFile 
{
	
	/**
	 * @param fileFullPath 文件路径
	 * @param pBuf 缓存
	 * @param pos 缓存文件起始位置
	 * @param size 读取字节数
	 * @param skip 跳跃字节数
	 * @return 真实读取字节数
	 * @throws IOException
	 */
	public static final int read(String fileFullPath, byte[] pBuf, int pos, int size, int skip)// throws IOException 
	{
		File file = new File(fileFullPath);
		if (file.exists()) 
		{
			try
			{

				FileInputStream fis = new FileInputStream(file);
				fis.skip(skip);
				int readSize = fis.read(pBuf, pos, size);
				fis.close();
				fis = null;
				return readSize;
			}
			catch(Exception ex)
			{
				ex.printStackTrace();
			}
			
		} 
		return 0;
	}

	/**
	 * @param fileFullPath 读取文件路径
	 * @return 返回整个文件数据
	 * @throws IOException 没有文件或者出异常返回null
	 */
	public static final byte[] readAll(String fileFullPath)// throws IOException 
	{
		File file = new File(fileFullPath);// MStore.instance.getFileStreamPath(fileFullPath);
		if (file.exists()) 
		{
			try
			{
				FileInputStream fis = new FileInputStream(file);// MStore.instance.openFileInput(fileFullPath);
				int readSize = fis.available();
				byte[] buff = new byte[readSize];
				fis.read(buff);
				fis.close();
				fis = null;
				return buff;
			}
			catch(Exception ex)
			{
				ex.printStackTrace();
			}
		}
		return null;
	}

	
	/**
	 * 多线程读写的时候需要注意，外部的方法需要同步
	 * @param fileFullPath 写入的文件路径
	 * @param pBuf 写入文件的缓存
	 * @param pos 缓存起始位置
	 * @param size 写入字节数
	 * @param skip 文件跳跃字节数
	 * @return 如果文件及路径不存在将创建文件
	 * @throws IOException
	 */
	public static final boolean write(String fileFullPath, byte[] pBuf, int pos, int size, int skip) //throws IOException 
	{
		File file = new File(fileFullPath);
		RandomAccessFile randomAccessFile = null;
		
		try
		{
			// 文件存在
			if (file.exists())
			{
				randomAccessFile = new RandomAccessFile(file, "rw");
			} 
			else // 文件不存在
			{
				// 判断父目录是否存在
				File p = file.getParentFile();
				boolean isOk = false;
				if (p != null && !p.exists())// 目录不存在创建
				{
					isOk = p.mkdirs();
				} 
				else if (p != null && p.exists())// 目录存在
				{
					isOk = true;
				}
				if (isOk) 
				{
					boolean isSucess = file.createNewFile();
					if (isSucess) 
					{
						randomAccessFile = new RandomAccessFile(file, "rw");
					}
				}
			}
			if (null != randomAccessFile)
			{
				randomAccessFile.seek(skip);
				randomAccessFile.write(pBuf, pos, size);
				randomAccessFile.close();
				return true;
			}
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		
		
		return false;
	}

	
	/**
	 * @param fileFullPath 文件路径
	 * @return 返回文件大小
	 * @throws IOException
	 */
	public static final int getSize(String fileFullPath)  
	{
		try
		{
			File file = new File(fileFullPath);
			if (file.exists()) 
			{
				int size = (int) file.length();
				return size;
			}
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		return 0;
	}
	
	/**
	 * @param fileFullPath 文件路径
	 * @return
	 */
	public static final boolean fileExist(String fileFullPath) 
	{
		File file = new File(fileFullPath);
		if (file.exists()) 
		{
			return true;
		}
		return false;
	}
	


	/**
	 * @param filePath
	 * @return 对于删除单个文件，成功返回true
	 */
	public static final boolean delFile(String filePath) 
	{
		if(filePath==null)
			return false;
		File file = new File(filePath);
		boolean ret = file.delete();
		return ret;
	}
	
	/**
	 * @param dir
	 * @return 删除文件夹或者文件
	 */
	public static final boolean delete(File dir)  
	{  
	    if (dir.isDirectory())  
	    {  
	        File[] listFiles = dir.listFiles();  
	        for (int i = 0; i < listFiles.length ; i++)  
	        {  
	        	 delete(listFiles[i]);
	        }  
	    }  
	    return dir.delete();  
	}  
	
	
	/**
	 * @param filePath  删除文件夹或者文件
	 */
	public static final void deleteAll(String filePath)  
	{  
		File dir = new File(filePath);
	    if (dir.isDirectory())  
	    {  
	        File[] listFiles = dir.listFiles();  
	        for (int i = 0; i < listFiles.length ; i++)  
	        {  
	        	delete(listFiles[i]);
	        }  
	    }  
	    else
	    {
	    	dir.delete();  
	    }
	}  
	
	
	/**
	 * @param fileFullPath
	 * @return 创建目录
	 */
	public static final boolean createDirectory(String fileFullPath )
	{
		boolean isOk = false;
		File file = new File(fileFullPath);
		isOk=file.exists();
		if(!isOk)
		isOk = file.mkdirs();
		return isOk;
	}
	
	
	/**
	 * @param filePath
	 * @return 返回路径下的目录及文件对象
	 */ 
	public static final File[] getDirectory(String filePath)
	{
		File dir = new File(filePath);
	    if (dir.isDirectory())  
	    {  
	        File[] listFiles = dir.listFiles();  
	        return listFiles;
	    }  
	    return  null;
	}
	
	
	/**
	 * @return 返回 手机可用空间
	 */
	public static final long phone_storage_free()
	{
	    File path = Environment.getDataDirectory();
	    StatFs stat = new StatFs(path.getPath());
	    long free_memory = stat.getAvailableBlocks() * stat.getBlockSize(); //return value is in bytes

	    return free_memory;
	}

	
	/**
	 * @return 返回已经使用的手机空间字节数
	 */
	public static final long phone_storage_used()
	{
	    File path = Environment.getDataDirectory();
	    StatFs stat = new StatFs(path.getPath());
	    long used_memory = (stat.getBlockCount() - stat.getAvailableBlocks()) * stat.getBlockSize(); //return value is in bytes

	    return used_memory;
	}

	/**
	 * @return 返回总的手机空间字节数
	 */
	public static final long phone_storage_total()
	{
	    File path = Environment.getDataDirectory();
	    StatFs stat = new StatFs(path.getPath());
	    long total_memory = stat.getBlockCount() * stat.getBlockSize(); //return value is in bytes

	    return total_memory;
	}   

	/**
	 * @return 返回总的卡的可用空间字节数
	 */
	public static final long sd_card_free()
	{

	    File path = Environment.getExternalStorageDirectory();
	    StatFs stat = new StatFs(path.getPath());
	    long free_memory = stat.getAvailableBlocks() * stat.getBlockSize(); //return value is in bytes

	    return free_memory;
	}
	
	/**
	 * @return 返回总的卡的已用空间字节数
	 */
	public static final long sd_card_used()
	{

	    File path = Environment.getExternalStorageDirectory();
	    StatFs stat = new StatFs(path.getPath());
	    long used_memory = (stat.getBlockCount() - stat.getAvailableBlocks()) * stat.getBlockSize(); //return value is in bytes

	    return used_memory;
	}
	
	/**
	 * @return 返回总的卡的总空间字节数
	 */
	public static final long sd_card_total()
	{

	    File path = Environment.getExternalStorageDirectory();
	    StatFs stat = new StatFs(path.getPath());
	    long total_memory = stat.getBlockCount() * stat.getBlockSize(); //return value is in bytes

	    return total_memory;
	}
	
	/**
	 * @return sd卡是否存在
	 */
	public static final boolean isSDCardExist()
	{
		if(android.os.Environment.getExternalStorageState().equals(android.os.Environment.MEDIA_MOUNTED))
		{
			return true;
		}
		return false;
	}
}
