package com.andframework.zmfile;

import android.os.Environment;



/**
 * @author 徐德良
 *该类使用的时候务必初始化approot应用根目录
 */
public class ZMFilePath 
{
	/**
	 * "/";分隔符
	 */
	public  static final String FILE_SEPARATOR = System.getProperty("file.separator");
	
	/**
	 * sd卡根目录
	 */
	private static  final String SDCARDROOT = Environment.getExternalStorageDirectory().getPath();
	
	/**
	 * 手机卡跟目录
	 */
	private static final String PHONEROOT = Environment.getDataDirectory().getPath();
	
	/**
	 * assets目录
	 */
	private static final String ASSETROOT = "file:///android_asset";
	
	
	/**
	 * 手机sdcard
	 */
	public static final byte CARD_ROOT_TYPE  = 0;
	
	/**
	 * 应用目录
	 */
	public static final byte PHONE_ROOT_TYPE = 1;
	
	/**
	 * asset目录
	 */
	public static final byte ASSET_ROOT_TYPE = 2;
	
	/**
	 * 根路径
	 */
	private byte  rootType = CARD_ROOT_TYPE;
	
	/**
	 * 程序启动初始化的时候需要初始化根路径，只能初始化一次
	 */
	public  static String APPROOT = ".sys";
	
	
	/**
	 * 使用应用跟目录，为了在非程序根目录下读取数据
	 */
	private boolean isUseAppRoot=true;
	
	
	/**
	 * 目录缓存
	 */
	private StringBuffer filePath = new StringBuffer();
	
	
	/**
	 * @param irootType 表示根目录 0表示手机根目录 1表示卡根目录 2表示asset根目录
	 * @param iisUseAppRoot  表示是否使用应用根目录，true表示使用，根据ZMFilePath.APPROOT 设置，false表示不使用
	 */
	public ZMFilePath(final byte irootType,final boolean iisUseAppRoot)
	{
		rootType=irootType;
		isUseAppRoot = iisUseAppRoot;
		if(rootType==CARD_ROOT_TYPE)
		{
			if(!ZMFile.isSDCardExist())
			{
				filePath.append(PHONEROOT);
			}
			else
			{
				filePath.append(SDCARDROOT);
				
			}
			filePath.append(FILE_SEPARATOR);
		}
		else if(rootType==PHONE_ROOT_TYPE)
		{
			filePath.append(PHONEROOT);
			filePath.append(FILE_SEPARATOR);
		}
		else if(rootType==ASSET_ROOT_TYPE)
		{
			filePath.append(ASSETROOT);
			filePath.append(FILE_SEPARATOR);
		}
		else
		{
			filePath.append(SDCARDROOT);
			filePath.append(FILE_SEPARATOR);
		}
		
		if(isUseAppRoot)
		{
			filePath.append(APPROOT);
			filePath.append(FILE_SEPARATOR);
		}
		
	}
	
	/**
	 * 默认卡目录，使用应用跟路径
	 */
	public ZMFilePath()
	{
		this(CARD_ROOT_TYPE,true);
		/*rootType=CARD_ROOT_TYPE;
		isUseAppRoot = true;
		
		filePath.append(SDCARDROOT);
		filePath.append(FILE_SEPARATOR);
		
		filePath.append(APPROOT);
		filePath.append(FILE_SEPARATOR);*/
	}
	
	
	/**
	 * @param pathNode 路径节点压栈
	 * @return
	 */
	public String pushPathNode(String pathNode)
	{
		if(pathNode!=null && !pathNode.trim().equals(""))
		{
			filePath.append(pathNode);
			filePath.append(FILE_SEPARATOR);
		}
		
		return filePath.toString();
	}
	
	
	/**
	 * @param pathNode 删除路径节点
	 * @return
	 */
	public boolean deletePathNode(String pathNode)
	{
		if(pathNode!=null && !pathNode.trim().equals(""))
		{
			String temStr = filePath.toString();
			if(temStr.contains(pathNode+FILE_SEPARATOR))
			{
				temStr = temStr.replace(pathNode+FILE_SEPARATOR, "");
				filePath =  new StringBuffer(temStr);
				return true;
			}
			else if(temStr.contains(pathNode))
			{
				temStr = temStr.replace(pathNode, "");
				filePath =  new StringBuffer(temStr);
				return true;
			}
		}
		return false;
	}
	
	/**
	 * @param fileName添加文件名字
	 * @return
	 */
	public String addFileName(String fileName)
	{
		filePath.append(fileName);
		return filePath.toString();
	}
	
	
	
	/**
	 * @return 路径出栈
	 */
	public String pullFilePath()
	{
		return filePath.toString();
		
	}
	
	/**
	 * @return 路径出栈
	 */
	public String toString()
	{
		return filePath.toString();
	}
}
